@@ -1,7 +1,5 @@
-#!/usr/bin/python
-
 # Audio Tools, a module and set of tools for manipulating audio data
-# Copyright (C) 2007-2014  Brian Langenberger
+# Copyright (C) 2007-2015  Brian Langenberger
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -108,7 +106,7 @@ class MP3Audio(AudioFile):
 
         AudioFile.__init__(self, filename)
 
-        from audiotools.bitstream import BitstreamReader
+        from audiotools.bitstream import parse
 
         try:
             mp3file = open(filename, "rb")
@@ -128,14 +126,15 @@ class MP3Audio(AudioFile):
              bit_rate,
              sample_rate,
              pad,
-             channels) = BitstreamReader(mp3file, 0).parse(
-                "11u 2u 2u 1p 4u 2u 1u 1p 2u 6p")
+             channels) = parse("11u 2u 2u 1p 4u 2u 1u 1p 2u 6p",
+                               False,
+                               mp3file.read(4))
 
             self.__samplerate__ = self.SAMPLE_RATE[mpeg_id][sample_rate]
-            if (self.__samplerate__ is None):
+            if self.__samplerate__ is None:
                 from audiotools.text import ERR_MP3_INVALID_SAMPLE_RATE
                 raise InvalidMP3(ERR_MP3_INVALID_SAMPLE_RATE)
-            if (channels in (0, 1, 2)):
+            if channels in (0, 1, 2):
                 self.__channels__ = 2
             else:
                 self.__channels__ = 1
@@ -146,19 +145,21 @@ class MP3Audio(AudioFile):
                                                          sample_rate,
                                                          pad) - 4)
 
-            if (("Xing" in first_frame) and
-                (len(first_frame[first_frame.index("Xing"):
-                                 first_frame.index("Xing") + 160]) == 160)):
+            if ((b"Xing" in first_frame) and
+                (len(first_frame[first_frame.index(b"Xing"):
+                                 first_frame.index(b"Xing") + 160]) == 160)):
                 # pull length from Xing header, if present
                 self.__pcm_frames__ = (
-                    BitstreamReader(
-                        first_frame[first_frame.index("Xing"):
-                                    first_frame.index("Xing") + 160],
-                        0).parse("32p 32p 32u 32p 832p")[0] *
+                    parse("32p 32p 32u 32p 832p",
+                          0,
+                          first_frame[first_frame.index(b"Xing"):
+                                      first_frame.index(b"Xing") + 160])[0] *
                     self.PCM_FRAMES_PER_MPEG_FRAME[layer])
             else:
                 # otherwise, bounce through file frames
-                reader = BitstreamReader(mp3file, 0)
+                from audiotools.bitstream import BitstreamReader
+
+                reader = BitstreamReader(mp3file, False)
                 self.__pcm_frames__ = 0
 
                 try:
@@ -169,7 +170,7 @@ class MP3Audio(AudioFile):
                      sample_rate,
                      pad) = reader.parse("11u 2u 2u 1p 4u 2u 1u 9p")
 
-                    while (frame_sync == 0x7FF):
+                    while frame_sync == 0x7FF:
                         self.__pcm_frames__ += \
                             self.PCM_FRAMES_PER_MPEG_FRAME[layer]
 
@@ -188,7 +189,7 @@ class MP3Audio(AudioFile):
                 except IOError:
                     pass
                 except ValueError as err:
-                    raise InvalidMP3(unicode(err))
+                    raise InvalidMP3(err)
         finally:
             mp3file.close()
 
@@ -197,6 +198,17 @@ class MP3Audio(AudioFile):
 
         return False
 
+    @classmethod
+    def supports_to_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .to_pcm() method"""
+
+        try:
+            from audiotools.decoders import MP3Decoder
+            return True
+        except ImportError:
+            return False
+
     def to_pcm(self):
         """returns a PCMReader object containing the track's PCM data"""
 
@@ -204,6 +216,17 @@ class MP3Audio(AudioFile):
 
         return MP3Decoder(self.filename)
 
+    @classmethod
+    def supports_from_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .from_pcm() classmethod"""
+
+        try:
+            from audiotools.encoders import encode_mp3
+            return True
+        except ImportError:
+            return False
+
     @classmethod
     def from_pcm(cls, filename, pcmreader,
                  compression=None, total_pcm_frames=None):
@@ -228,18 +251,17 @@ class MP3Audio(AudioFile):
             compression = __default_quality__(cls.NAME)
 
         try:
-            if (total_pcm_frames is not None):
+            if total_pcm_frames is not None:
                 from audiotools import CounterPCMReader
                 pcmreader = CounterPCMReader(pcmreader)
 
             encode_mp3(filename,
-                       BufferedPCMReader(
-                           PCMConverter(pcmreader,
-                                        sample_rate=pcmreader.sample_rate,
-                                        channels=min(pcmreader.channels, 2),
-                                        channel_mask=ChannelMask.from_channels(
-                                            min(pcmreader.channels, 2)),
-                                        bits_per_sample=16)),
+                       PCMConverter(pcmreader,
+                                    sample_rate=pcmreader.sample_rate,
+                                    channels=min(pcmreader.channels, 2),
+                                    channel_mask=ChannelMask.from_channels(
+                                        min(pcmreader.channels, 2)),
+                                    bits_per_sample=16),
                        compression)
 
             if ((total_pcm_frames is not None) and
@@ -252,6 +274,8 @@ class MP3Audio(AudioFile):
         except (ValueError, IOError) as err:
             cls.__unlink__(filename)
             raise EncodingError(str(err))
+        finally:
+            pcmreader.close()
 
     def bits_per_sample(self):
         """returns an integer number of bits-per-sample this track contains"""
@@ -283,15 +307,13 @@ class MP3Audio(AudioFile):
         from audiotools.id3 import read_id3v2_comment
         from audiotools.id3v1 import ID3v1Comment
 
-        f = open(self.filename, "rb")
-        try:
-            if (f.read(3) == "ID3"):
+        with open(self.filename, "rb") as f:
+            if f.read(3) == b"ID3":
                 id3v2 = read_id3v2_comment(self.filename)
 
                 try:
                     # yes IDv2, yes ID3v1
-                    return ID3CommentPair(id3v2,
-                                          ID3v1Comment.parse(f))
+                    return ID3CommentPair(id3v2, ID3v1Comment.parse(f))
                 except ValueError:
                     # yes ID3v2, no ID3v1
                     return id3v2
@@ -302,8 +324,6 @@ class MP3Audio(AudioFile):
                 except ValueError:
                     # no ID3v2, no ID3v1
                     return None
-        finally:
-            f.close()
 
     def update_metadata(self, metadata):
         """takes this track's current MetaData object
@@ -321,14 +341,14 @@ class MP3Audio(AudioFile):
         from audiotools.id3v1 import ID3v1Comment
         from audiotools.bitstream import BitstreamWriter
 
-        if (metadata is None):
+        if metadata is None:
             return
         elif (not (isinstance(metadata, ID3v2Comment) or
                    isinstance(metadata, ID3CommentPair) or
                    isinstance(metadata, ID3v1Comment))):
             from audiotools.text import ERR_FOREIGN_METADATA
             raise ValueError(ERR_FOREIGN_METADATA)
-        elif (not os.access(self.filename, os.W_OK)):
+        elif not os.access(self.filename, os.W_OK):
             raise IOError(self.filename)
 
         new_mp3 = TemporaryFile(self.filename)
@@ -343,14 +363,14 @@ class MP3Audio(AudioFile):
         old_mp3 = LimitedFileReader(old_mp3, data_end - data_start)
 
         # write id3v2 + data + id3v1 to file
-        if (isinstance(metadata, ID3CommentPair)):
-            metadata.id3v2.build(BitstreamWriter(new_mp3, 0))
+        if isinstance(metadata, ID3CommentPair):
+            metadata.id3v2.build(BitstreamWriter(new_mp3, False))
             transfer_data(old_mp3.read, new_mp3.write)
             metadata.id3v1.build(new_mp3)
-        elif (isinstance(metadata, ID3v2Comment)):
-            metadata.build(BitstreamWriter(new_mp3, 0))
+        elif isinstance(metadata, ID3v2Comment):
+            metadata.build(BitstreamWriter(new_mp3, False))
             transfer_data(old_mp3.read, new_mp3.write)
-        elif (isinstance(metadata, ID3v1Comment)):
+        elif isinstance(metadata, ID3v1Comment):
             transfer_data(old_mp3.read, new_mp3.write)
             metadata.build(new_mp3)
 
@@ -371,7 +391,7 @@ class MP3Audio(AudioFile):
         from audiotools.id3 import ID3CommentPair
         from audiotools.id3v1 import ID3v1Comment
 
-        if (metadata is None):
+        if metadata is None:
             return self.delete_metadata()
 
         if (not (isinstance(metadata, ID3v2Comment) or
@@ -394,14 +414,14 @@ class MP3Audio(AudioFile):
                                                                 "id3v1",
                                                                 DEFAULT_ID3V1),
                                              DEFAULT_ID3V1)
-            if ((id3v2_class is not None) and (id3v1_class is not None)):
+            if (id3v2_class is not None) and (id3v1_class is not None):
                 self.update_metadata(
                     ID3CommentPair.converted(metadata,
                                              id3v2_class=id3v2_class,
                                              id3v1_class=id3v1_class))
-            elif (id3v2_class is not None):
+            elif id3v2_class is not None:
                 self.update_metadata(id3v2_class.converted(metadata))
-            elif (id3v1_class is not None):
+            elif id3v1_class is not None:
                 self.update_metadata(id3v1_class.converted(metadata))
             else:
                 return
@@ -422,7 +442,7 @@ class MP3Audio(AudioFile):
         # this works a lot like update_metadata
         # but without any new metadata to set
 
-        if (not os.access(self.filename, os.W_OK)):
+        if not os.access(self.filename, os.W_OK):
             raise IOError(self.filename)
 
         new_mp3 = TemporaryFile(self.filename)
@@ -461,19 +481,20 @@ class MP3Audio(AudioFile):
         from audiotools import open as open_audiofile
         from audiotools.text import CLEAN_REMOVE_DUPLICATE_ID3V2
 
-        if (total_id3v2_comments(open(self.filename, "rb")) > 1):
-            file_fixes = [CLEAN_REMOVE_DUPLICATE_ID3V2]
-        else:
-            file_fixes = []
+        with open(self.filename, "rb") as f:
+            if total_id3v2_comments(f) > 1:
+                file_fixes = [CLEAN_REMOVE_DUPLICATE_ID3V2]
+            else:
+                file_fixes = []
 
-        if (output_filename is None):
+        if output_filename is None:
             # dry run only
             metadata = self.get_metadata()
-            if (metadata is not None):
+            if metadata is not None:
                 (metadata, fixes) = metadata.clean()
                 return file_fixes + fixes
             else:
-                return []
+                return file_fixes
         else:
             # perform complete fix
             input_f = open(self.filename, "rb")
@@ -486,14 +507,14 @@ class MP3Audio(AudioFile):
 
             new_track = open_audiofile(output_filename)
             metadata = self.get_metadata()
-            if (metadata is not None):
+            if metadata is not None:
                 (metadata, fixes) = metadata.clean()
-                if (len(file_fixes + fixes) > 0):
+                if len(file_fixes + fixes) > 0:
                     # only update metadata if fixes are actually performed
                     new_track.update_metadata(metadata)
                 return file_fixes + fixes
             else:
-                return []
+                return file_fixes
 
     # places mp3file at the position of the next MP3 frame's start
     @classmethod
@@ -506,34 +527,30 @@ class MP3Audio(AudioFile):
         # then find the next mp3 frame
         from audiotools.bitstream import BitstreamReader
 
-        reader = BitstreamReader(mp3file, 0)
-        reader.mark()
+        reader = BitstreamReader(mp3file, False)
+        pos = reader.getpos()
         try:
             (sync,
              mpeg_id,
              layer_description) = reader.parse("11u 2u 2u 1p")
         except IOError as err:
-            reader.unmark()
             raise err
 
         while (not ((sync == 0x7FF) and
                     (mpeg_id in (0, 2, 3)) and
                     (layer_description in (1, 2, 3)))):
-            reader.rewind()
-            reader.unmark()
+            reader.setpos(pos)
             reader.skip(8)
             bytes_skipped += 1
-            reader.mark()
+            pos = reader.getpos()
             try:
                 (sync,
                  mpeg_id,
                  layer_description) = reader.parse("11u 2u 2u 1p")
             except IOError as err:
-                reader.unmark()
                 raise err
         else:
-            reader.rewind()
-            reader.unmark()
+            reader.setpos(pos)
             return bytes_skipped
 
     @classmethod
@@ -547,20 +564,18 @@ class MP3Audio(AudioFile):
 
         from audiotools.bitstream import BitstreamReader
 
-        reader = BitstreamReader(mp3file, 0)
+        reader = BitstreamReader(mp3file, False)
 
         # skip over any bytes that aren't a valid MPEG header
-        reader.mark()
+        pos = reader.getpos()
         (frame_sync, mpeg_id, layer) = reader.parse("11u 2u 2u 1p")
         while (not ((frame_sync == 0x7FF) and
                     (mpeg_id in (0, 2, 3)) and
                     (layer in (1, 2, 3)))):
-            reader.rewind()
-            reader.unmark()
+            reader.setpos(pos)
             reader.skip(8)
-            reader.mark()
-        reader.rewind()
-        reader.unmark()
+            pos = reader.getpos()
+        reader.setpos(pos)
 
     @classmethod
     def __find_last_mp3_frame__(cls, mp3file):
@@ -573,7 +588,7 @@ class MP3Audio(AudioFile):
         """
 
         mp3file.seek(-128, 2)
-        if (mp3file.read(3) == 'TAG'):
+        if mp3file.read(3) == b'TAG':
             mp3file.seek(-128, 2)
             return
         else:
@@ -592,14 +607,14 @@ class MP3Audio(AudioFile):
         """
 
         sample_rate = self.SAMPLE_RATE[mpeg_id][sample_rate]
-        if (sample_rate is None):
+        if sample_rate is None:
             from audiotools.text import ERR_MP3_INVALID_SAMPLE_RATE
             raise ValueError(ERR_MP3_INVALID_SAMPLE_RATE)
         bit_rate = self.BIT_RATE[mpeg_id][layer][bit_rate]
-        if (bit_rate is None):
+        if bit_rate is None:
             from audiotools.text import ERR_MP3_INVALID_BIT_RATE
             raise ValueError(ERR_MP3_INVALID_BIT_RATE)
-        if (layer == 3):  # layer I
+        if layer == 3:  # layer I
             return (((12 * bit_rate) // sample_rate) + pad) * 4
         else:             # layer II/III
             return ((144 * bit_rate) // sample_rate) + pad
@@ -609,52 +624,6 @@ class MP3Audio(AudioFile):
 
         return self.__pcm_frames__
 
-    @classmethod
-    def available(cls, system_binaries):
-        """returns True if all necessary compenents are available
-        to support format"""
-
-        try:
-            from audiotools.decoders import MP3Decoder
-            from audiotools.encoders import encode_mp3
-
-            return True
-        except ImportError:
-            return False
-
-    @classmethod
-    def missing_components(cls, messenger):
-        """given a Messenger object, displays missing binaries or libraries
-        needed to support this format and where to get them"""
-
-        from audiotools.text import (ERR_LIBRARY_NEEDED,
-                                     ERR_LIBRARY_DOWNLOAD_URL,
-                                     ERR_PROGRAM_PACKAGE_MANAGER)
-
-        format_ = cls.NAME.decode('ascii')
-
-        # display where to get libmp3lame
-        messenger.info(
-            ERR_LIBRARY_NEEDED %
-            {"library": u"\"libmp3lame\"",
-             "format": format_})
-        messenger.info(
-            ERR_LIBRARY_DOWNLOAD_URL %
-            {"library": u"mp3lame",
-             "url": "http://lame.sourceforge.net/"})
-
-        # then display where to get libmpg123
-        messenger.info(
-            ERR_LIBRARY_NEEDED %
-            {"library": u"\"libmpg123\"",
-             "format": format_})
-        messenger.info(
-            ERR_LIBRARY_DOWNLOAD_URL %
-            {"library": u"mpg123",
-             "url": u"http://www.mpg123.org/"})
-
-        messenger.info(ERR_PROGRAM_PACKAGE_MANAGER)
-
 
 class MP2Audio(MP3Audio):
     """an MP2 audio file"""
@@ -671,6 +640,17 @@ class MP2Audio(MP3Audio):
     COMPRESSION_DESCRIPTIONS = {"64": COMP_TWOLAME_64,
                                 "384": COMP_TWOLAME_384}
 
+    @classmethod
+    def supports_from_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .from_pcm() classmethod"""
+
+        try:
+            from audiotools.encoders import encode_mp2
+            return True
+        except ImportError:
+            return False
+
     @classmethod
     def from_pcm(cls, filename, pcmreader,
                  compression=None, total_pcm_frames=None):
@@ -695,7 +675,7 @@ class MP2Audio(MP3Audio):
              (compression not in cls.COMPRESSION_MODES))):
             compression = __default_quality__(cls.NAME)
 
-        if (pcmreader.sample_rate in (32000, 48000, 44100)):
+        if pcmreader.sample_rate in (32000, 48000, 44100):
             sample_rate = pcmreader.sample_rate
         else:
             sample_rate = [32000,
@@ -706,19 +686,18 @@ class MP2Audio(MP3Audio):
                                                  48000],
                                                 pcmreader.sample_rate)]
 
-        try:
-            if (total_pcm_frames is not None):
-                from audiotools import CounterPCMReader
-                pcmreader = CounterPCMReader(pcmreader)
+        if total_pcm_frames is not None:
+            from audiotools import CounterPCMReader
+            pcmreader = CounterPCMReader(pcmreader)
 
+        try:
             encode_mp2(filename,
-                       BufferedPCMReader(
-                           PCMConverter(pcmreader,
-                                        sample_rate=sample_rate,
-                                        channels=min(pcmreader.channels, 2),
-                                        channel_mask=ChannelMask.from_channels(
-                                            min(pcmreader.channels, 2)),
-                                        bits_per_sample=16)),
+                       PCMConverter(pcmreader,
+                                    sample_rate=sample_rate,
+                                    channels=min(pcmreader.channels, 2),
+                                    channel_mask=ChannelMask.from_channels(
+                                        min(pcmreader.channels, 2)),
+                                    bits_per_sample=16),
                        int(compression))
 
             if ((total_pcm_frames is not None) and
@@ -731,49 +710,5 @@ class MP2Audio(MP3Audio):
         except (ValueError, IOError) as err:
             cls.__unlink__(filename)
             raise EncodingError(str(err))
-
-    @classmethod
-    def available(cls, system_binaries):
-        """returns True if all necessary compenents are available
-        to support format"""
-
-        try:
-            from audiotools.decoders import MP3Decoder
-            from audiotools.encoders import encode_mp2
-
-            return True
-        except ImportError:
-            return False
-
-    @classmethod
-    def missing_components(cls, messenger):
-        """given a Messenger object, displays missing binaries or libraries
-        needed to support this format and where to get them"""
-
-        from audiotools.text import (ERR_LIBRARY_NEEDED,
-                                     ERR_LIBRARY_DOWNLOAD_URL,
-                                     ERR_PROGRAM_PACKAGE_MANAGER)
-
-        format_ = cls.NAME.decode('ascii')
-
-        # display where to get libtwo,ame
-        messenger.info(
-            ERR_LIBRARY_NEEDED %
-            {"library": u"\"libtwolame\"",
-             "format": format_})
-        messenger.info(
-            ERR_LIBRARY_DOWNLOAD_URL %
-            {"library": u"twolame",
-             "url": "http://twolame.sourceforge.net/"})
-
-        # then display where to get libmpg123
-        messenger.info(
-            ERR_LIBRARY_NEEDED %
-            {"library": u"\"libmpg123\"",
-             "format": format_})
-        messenger.info(
-            ERR_LIBRARY_DOWNLOAD_URL %
-            {"library": u"mpg123",
-             "url": u"http://www.mpg123.org/"})
-
-        messenger.info(ERR_PROGRAM_PACKAGE_MANAGER)
+        finally:
+            pcmreader.close()
