@@ -1,7 +1,5 @@
-#!/usr/bin/python
-
 # Audio Tools, a module and set of tools for manipulating audio data
-# Copyright (C) 2007-2014  Brian Langenberger
+# Copyright (C) 2007-2015  Brian Langenberger
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,22 +18,22 @@
 from audiotools import iter_last
 from audiotools.bitstream import BitstreamReader
 from audiotools.bitstream import HuffmanTree
-from audiotools.pcm import from_list, from_channels
+from audiotools.pcm import empty_framelist, from_list, from_channels
 from operator import concat
 
 
 def log2(i):
     bits = -1
-    while (i):
+    while i:
         bits += 1
         i >>= 1
     return bits
 
 
 def sign_only(value):
-    if (value == 0):
+    if value == 0:
         return 0
-    elif (value > 0):
+    elif value > 0:
         return 1
     else:
         return -1
@@ -46,7 +44,7 @@ def truncate_bits(value, bits):
     truncated = value & ((1 << bits) - 1)
 
     # apply newly created sign bit
-    if (truncated & (1 << (bits - 1))):
+    if truncated & (1 << (bits - 1)):
         return truncated - (1 << bits)
     else:
         return truncated
@@ -66,68 +64,65 @@ RESIDUAL = HuffmanTree([[0], 0,
 
 class ALACDecoder(object):
     def __init__(self, filename):
-        self.reader = BitstreamReader(open(filename, "rb"), 0)
+        self.reader = BitstreamReader(open(filename, "rb"), False)
 
-        self.reader.mark()
+        stream_start = self.reader.getpos()
+
+        # locate the "alac" atom
+        # which is full of required decoding parameters
         try:
-            # locate the "alac" atom
-            # which is full of required decoding parameters
-            try:
-                stsd = self.find_sub_atom("moov", "trak", "mdia",
-                                          "minf", "stbl", "stsd")
-            except KeyError:
-                raise ValueError("required stsd atom not found")
+            stsd = self.find_sub_atom(b"moov", b"trak", b"mdia",
+                                      b"minf", b"stbl", b"stsd")
+        except KeyError:
+            raise ValueError("required stsd atom not found")
 
-            (stsd_version, descriptions) = stsd.parse("8u 24p 32u")
-            (alac1,
-             alac2,
-             self.samples_per_frame,
-             self.bits_per_sample,
-             self.history_multiplier,
-             self.initial_history,
-             self.maximum_k,
-             self.channels,
-             self.sample_rate) = stsd.parse(
-                # ignore much of the stuff in the "high" ALAC atom
-                "32p 4b 6P 16p 16p 16p 4P 16p 16p 16p 16p 4P" +
-                # and use the attributes in the "low" ALAC atom instead
-                "32p 4b 4P 32u 8p 8u 8u 8u 8u 8u 16p 32p 32p 32u")
+        (stsd_version, descriptions) = stsd.parse("8u 24p 32u")
+        (alac1,
+         alac2,
+         self.samples_per_frame,
+         self.bits_per_sample,
+         self.history_multiplier,
+         self.initial_history,
+         self.maximum_k,
+         self.channels,
+         self.sample_rate) = stsd.parse(
+            # ignore much of the stuff in the "high" ALAC atom
+            "32p 4b 6P 16p 16p 16p 4P 16p 16p 16p 16p 4P" +
+            # and use the attributes in the "low" ALAC atom instead
+            "32p 4b 4P 32u 8p 8u 8u 8u 8u 8u 16p 32p 32p 32u")
 
-            self.channel_mask = {1: 0x0004,
-                                 2: 0x0003,
-                                 3: 0x0007,
-                                 4: 0x0107,
-                                 5: 0x0037,
-                                 6: 0x003F,
-                                 7: 0x013F,
-                                 8: 0x00FF}.get(self.channels, 0)
+        self.channel_mask = {1: 0x0004,
+                             2: 0x0003,
+                             3: 0x0007,
+                             4: 0x0107,
+                             5: 0x0037,
+                             6: 0x003F,
+                             7: 0x013F,
+                             8: 0x00FF}.get(self.channels, 0)
 
-            if ((alac1 != 'alac') or (alac2 != 'alac')):
-                raise ValueError("Invalid alac atom")
+        if (alac1 != b'alac') or (alac2 != b'alac'):
+            raise ValueError("Invalid alac atom")
 
-            # also locate the "mdhd" atom
-            # which contains the stream's length in PCM frames
-            self.reader.rewind()
-            mdhd = self.find_sub_atom("moov", "trak", "mdia", "mdhd")
-            (version, ) = mdhd.parse("8u 24p")
-            if (version == 0):
-                (self.total_pcm_frames,) = mdhd.parse(
-                    "32p 32p 32p 32u 2P 16p")
-            elif (version == 1):
-                (self.total_pcm_frames,) = mdhd.parse(
-                    "64p 64p 32p 64U 2P 16p")
-            else:
-                raise ValueError("invalid mdhd version")
+        # also locate the "mdhd" atom
+        # which contains the stream's length in PCM frames
+        self.reader.setpos(stream_start)
+        mdhd = self.find_sub_atom(b"moov", b"trak", b"mdia", b"mdhd")
+        (version, ) = mdhd.parse("8u 24p")
+        if version == 0:
+            (self.total_pcm_frames,) = mdhd.parse(
+                "32p 32p 32p 32u 2P 16p")
+        elif version == 1:
+            (self.total_pcm_frames,) = mdhd.parse(
+                "64p 64p 32p 64U 2P 16p")
+        else:
+            raise ValueError("invalid mdhd version")
 
-            # finally, set our stream to the "mdat" atom
-            self.reader.rewind()
+        # finally, set our stream to the "mdat" atom
+        self.reader.setpos(stream_start)
+        (atom_size, atom_name) = self.reader.parse("32u 4b")
+        while atom_name != b"mdat":
+            self.reader.skip_bytes(atom_size - 8)
             (atom_size, atom_name) = self.reader.parse("32u 4b")
-            while (atom_name != "mdat"):
-                self.reader.skip_bytes(atom_size - 8)
-                (atom_size, atom_name) = self.reader.parse("32u 4b")
-
-        finally:
-            self.reader.unmark()
 
     def find_sub_atom(self, *atom_names):
         reader = self.reader
@@ -135,10 +130,10 @@ class ALACDecoder(object):
         for (last, next_atom) in iter_last(iter(atom_names)):
             try:
                 (length, stream_atom) = reader.parse("32u 4b")
-                while (stream_atom != next_atom):
+                while stream_atom != next_atom:
                     reader.skip_bytes(length - 8)
                     (length, stream_atom) = reader.parse("32u 4b")
-                if (last):
+                if last:
                     return reader.substream(length - 8)
                 else:
                     reader = reader.substream(length - 8)
@@ -147,43 +142,43 @@ class ALACDecoder(object):
 
     def read(self, pcm_frames):
         # if the stream is exhausted, return an empty pcm.FrameList object
-        if (self.total_pcm_frames == 0):
-            return from_list([], self.channels, self.bits_per_sample, True)
+        if self.total_pcm_frames == 0:
+            return empty_framelist(self.channels, self.bits_per_sample)
 
         # otherwise, read one ALAC frameset's worth of frame data
         frameset_data = []
         frame_channels = self.reader.read(3) + 1
-        while (frame_channels != 0x8):
+        while frame_channels != 0x8:
             frameset_data.extend(self.read_frame(frame_channels))
             frame_channels = self.reader.read(3) + 1
         self.reader.byte_align()
 
         # reorder the frameset to Wave order, depending on channel count
-        if ((self.channels == 1) or (self.channels == 2)):
+        if (self.channels == 1) or (self.channels == 2):
             pass
-        elif (self.channels == 3):
+        elif self.channels == 3:
             frameset_data = [frameset_data[1],
                              frameset_data[2],
                              frameset_data[0]]
-        elif (self.channels == 4):
+        elif self.channels == 4:
             frameset_data = [frameset_data[1],
                              frameset_data[2],
                              frameset_data[0],
                              frameset_data[3]]
-        elif (self.channels == 5):
+        elif self.channels == 5:
             frameset_data = [frameset_data[1],
                              frameset_data[2],
                              frameset_data[0],
                              frameset_data[3],
                              frameset_data[4]]
-        elif (self.channels == 6):
+        elif self.channels == 6:
             frameset_data = [frameset_data[1],
                              frameset_data[2],
                              frameset_data[0],
                              frameset_data[5],
                              frameset_data[3],
                              frameset_data[4]]
-        elif (self.channels == 7):
+        elif self.channels == 7:
             frameset_data = [frameset_data[1],
                              frameset_data[2],
                              frameset_data[0],
@@ -191,7 +186,7 @@ class ALACDecoder(object):
                              frameset_data[3],
                              frameset_data[4],
                              frameset_data[5]]
-        elif (self.channels == 8):
+        elif self.channels == 8:
             frameset_data = [frameset_data[3],
                              frameset_data[4],
                              frameset_data[0],
@@ -223,12 +218,12 @@ class ALACDecoder(object):
         has_sample_count = self.reader.read(1)
         uncompressed_lsb_size = self.reader.read(2)
         uncompressed = self.reader.read(1)
-        if (has_sample_count):
+        if has_sample_count:
             sample_count = self.reader.read(32)
         else:
             sample_count = self.samples_per_frame
 
-        if (uncompressed == 1):
+        if uncompressed == 1:
             # if the frame is uncompressed,
             # read the raw, interlaced samples
             samples = [self.reader.read_signed(self.bits_per_sample)
@@ -245,7 +240,7 @@ class ALACDecoder(object):
                                 for i in range(channel_count)]
 
             # optional uncompressed LSB values
-            if (uncompressed_lsb_size > 0):
+            if uncompressed_lsb_size > 0:
                 uncompressed_lsbs = [
                     self.reader.read(uncompressed_lsb_size * 8)
                     for i in range(sample_count * channel_count)]
@@ -279,7 +274,7 @@ class ALACDecoder(object):
 
             # if uncompressed LSB values are present,
             # prepend them to each sample of each channel
-            if (uncompressed_lsb_size > 0):
+            if uncompressed_lsb_size > 0:
                 channels = []
                 for (i, channel) in enumerate(decorrelated_channels):
                     assert(len(channel) ==
@@ -307,7 +302,7 @@ class ALACDecoder(object):
         sign_modifier = 0
         i = 0
 
-        while (i < sample_count):
+        while i < sample_count:
             # get an unsigned residual based on "history"
             # and on "sample_size" as a lst resort
             k = min(log2(history // (2 ** 9) + 3), self.maximum_k)
@@ -318,13 +313,13 @@ class ALACDecoder(object):
             sign_modifier = 0
 
             # change unsigned residual to signed residual
-            if (unsigned & 1):
+            if unsigned & 1:
                 residuals.append(-((unsigned + 1) // 2))
             else:
                 residuals.append(unsigned // 2)
 
             # update history based on unsigned residual
-            if (unsigned <= 0xFFFF):
+            if unsigned <= 0xFFFF:
                 history += ((unsigned * self.history_multiplier) -
                             ((history * self.history_multiplier) >> 9))
             else:
@@ -332,19 +327,19 @@ class ALACDecoder(object):
 
             # if history gets too small, we may have a block of 0 samples
             # which can be compressed more efficiently
-            if ((history < 128) and ((i + 1) < sample_count)):
+            if (history < 128) and ((i + 1) < sample_count):
                 zeroes_k = min(7 -
                                log2(history) +
                                ((history + 16) // 64),
                                self.maximum_k)
                 zero_residuals = self.read_residual(zeroes_k, 16)
-                if (zero_residuals > 0):
+                if zero_residuals > 0:
                     residuals.extend([0] * zero_residuals)
                     i += zero_residuals
 
                 history = 0
 
-                if (zero_residuals <= 0xFFFF):
+                if zero_residuals <= 0xFFFF:
                     sign_modifier = 1
 
             i += 1
@@ -353,15 +348,15 @@ class ALACDecoder(object):
 
     def read_residual(self, k, sample_size):
         msb = self.reader.read_huffman_code(RESIDUAL)
-        if (msb == -1):
+        if msb == -1:
             return self.reader.read(sample_size)
-        elif (k == 0):
+        elif k == 0:
             return msb
         else:
             lsb = self.reader.read(k)
-            if (lsb > 1):
+            if lsb > 1:
                 return msb * ((1 << k) - 1) + (lsb - 1)
-            elif (lsb == 1):
+            elif lsb == 1:
                 self.reader.unread(1)
                 return msb * ((1 << k) - 1)
             else:
@@ -373,7 +368,7 @@ class ALACDecoder(object):
         # first sample is always copied verbatim
         samples = [residuals.pop(0)]
 
-        if (len(qlp_coefficients) < 31):
+        if len(qlp_coefficients) < 31:
             # the next "coefficient count" samples
             # are applied as differences to the previous
             for i in range(len(qlp_coefficients)):
@@ -394,10 +389,10 @@ class ALACDecoder(object):
                 buf = samples[-len(qlp_coefficients) - 2:-1]
 
                 # error value then adjusts the coefficients table
-                if (residual > 0):
+                if residual > 0:
                     predictor_num = len(qlp_coefficients) - 1
 
-                    while ((predictor_num >= 0) and residual > 0):
+                    while (predictor_num >= 0) and residual > 0:
                         val = (buf[0] -
                                buf[len(qlp_coefficients) - predictor_num])
 
@@ -412,11 +407,11 @@ class ALACDecoder(object):
 
                         predictor_num -= 1
 
-                elif (residual < 0):
+                elif residual < 0:
                     # the same as above, but we break if residual goes positive
                     predictor_num = len(qlp_coefficients) - 1
 
-                    while ((predictor_num >= 0) and residual < 0):
+                    while (predictor_num >= 0) and (residual < 0):
                         val = (buf[0] -
                                buf[len(qlp_coefficients) - predictor_num])
 
@@ -440,9 +435,9 @@ class ALACDecoder(object):
 
     def decorrelate_channels(self, channel_data,
                              interlacing_shift, interlacing_leftweight):
-        if (len(channel_data) != 2):
+        if len(channel_data) != 2:
             return channel_data
-        elif (interlacing_leftweight == 0):
+        elif interlacing_leftweight == 0:
             return channel_data
         else:
             left = []
@@ -454,4 +449,10 @@ class ALACDecoder(object):
             return [left, right]
 
     def close(self):
-        pass
+        self.reader.close()
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
