@@ -1,7 +1,5 @@
-#!/usr/bin/python
-
 # Audio Tools, a module and set of tools for manipulating audio data
-# Copyright (C) 2007-2014  Brian Langenberger
+# Copyright (C) 2007-2015  Brian Langenberger
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -39,7 +37,8 @@ class VorbisComment(MetaData):
                      'year': u'DATE',
                      'album_number': u'DISCNUMBER',
                      'album_total': u'DISCTOTAL',
-                     'comment': u'COMMENT'}
+                     'comment': u'COMMENT',
+                     'compilation': u'COMPILATION'}
 
     ALIASES = {}
 
@@ -56,6 +55,13 @@ class VorbisComment(MetaData):
 
         vendor_string is a unicode string"""
 
+        from audiotools import PY3
+
+        # some debug type checking
+        for s in comment_strings:
+            assert(isinstance(s, str if PY3 else unicode))
+        assert(isinstance(vendor_string, str if PY3 else unicode))
+
         MetaData.__setattr__(self, "comment_strings", comment_strings)
         MetaData.__setattr__(self, "vendor_string", vendor_string)
 
@@ -71,6 +77,9 @@ class VorbisComment(MetaData):
         return [(key, self[key]) for key in self.keys()]
 
     def __contains__(self, key):
+        from audiotools import PY3
+        assert(isinstance(key, str if PY3 else unicode))
+
         matching_keys = self.ALIASES.get(key.upper(), frozenset([key.upper()]))
 
         return len([item_value for (item_key, item_value) in
@@ -79,6 +88,9 @@ class VorbisComment(MetaData):
                     if (item_key.upper() in matching_keys)]) > 0
 
     def __getitem__(self, key):
+        from audiotools import PY3
+        assert(isinstance(key, str if PY3 else unicode))
+
         matching_keys = self.ALIASES.get(key.upper(), frozenset([key.upper()]))
 
         values = [item_value for (item_key, item_value) in
@@ -86,20 +98,25 @@ class VorbisComment(MetaData):
                    if (u"=" in comment)]
                   if (item_key.upper() in matching_keys)]
 
-        if (len(values) > 0):
+        if len(values) > 0:
             return values
         else:
             raise KeyError(key)
 
     def __setitem__(self, key, values):
+        from audiotools import PY3
+        assert(isinstance(key, str if PY3 else unicode))
+        for v in values:
+            assert(isinstance(v, str if PY3 else unicode))
+
         new_values = values[:]
         new_comment_strings = []
         matching_keys = self.ALIASES.get(key.upper(), frozenset([key.upper()]))
 
         for comment in self.comment_strings:
-            if (u"=" in comment):
+            if u"=" in comment:
                 (c_key, c_value) = comment.split(u"=", 1)
-                if (c_key.upper() in matching_keys):
+                if c_key.upper() in matching_keys:
                     try:
                         # replace current value with newly set value
                         new_comment_strings.append(
@@ -121,13 +138,16 @@ class VorbisComment(MetaData):
         MetaData.__setattr__(self, "comment_strings", new_comment_strings)
 
     def __delitem__(self, key):
+        from audiotools import PY3
+        assert(isinstance(key, str if PY3 else unicode))
+
         new_comment_strings = []
         matching_keys = self.ALIASES.get(key.upper(), frozenset([key.upper()]))
 
         for comment in self.comment_strings:
-            if (u"=" in comment):
+            if u"=" in comment:
                 (c_key, c_value) = comment.split(u"=", 1)
-                if (c_key.upper() not in matching_keys):
+                if c_key.upper() not in matching_keys:
                     # passthrough unmatching values
                     new_comment_strings.append(comment)
             else:
@@ -162,7 +182,7 @@ class VorbisComment(MetaData):
         for comment in self.comment_strings:
             row = table.row()
 
-            if (u"=" in comment):
+            if u"=" in comment:
                 (tag, value) = comment.split(u"=", 1)
                 row.add_column(tag, "right")
                 row.add_column(u"=")
@@ -173,60 +193,74 @@ class VorbisComment(MetaData):
                 row.add_column(u"")
 
         return (u"%s:  %s" % (self.__comment_name__(),
-                              self.vendor_string) + linesep.decode('ascii') +
-                linesep.decode('ascii').join(table.format()))
+                              self.vendor_string) + linesep +
+                linesep.join(table.format()))
 
     def __getattr__(self, attr):
         # returns the first matching key for the given attribute
         # in our list of comment strings
 
-        if ((attr == "track_number") or (attr == "album_number")):
-            try:
-                # get the TRACKNUMBER/DISCNUMBER values
-                # return the first value that contains an integer
-                for value in self[self.ATTRIBUTE_MAP[attr]]:
-                    integer = re.search(r'\d+', value)
-                    if (integer is not None):
-                        return int(integer.group(0))
-                else:
-                    # otherwise, return None
-                    return None
-            except KeyError:
-                # if no TRACKNUMBER/DISCNUMBER, return None
-                return None
-        elif ((attr == "track_total") or (attr == "album_total")):
-            try:
-                # get the TRACKTOTAL/DISCTOTAL values
-                # return the first value that contains an integer
-                for value in self[self.ATTRIBUTE_MAP[attr]]:
-                    integer = re.search(r'\d+', value)
-                    if (integer is not None):
-                        return int(integer.group(0))
-            except KeyError:
-                pass
+        if attr in self.ATTRIBUTE_MAP:
+            key = self.ATTRIBUTE_MAP[attr]
 
-            # if no TRACKTOTAL/DISCTOTAL,
-            # or none of them contain an integer,
-            # look for slashed TRACKNUMBER/DISCNUMBER values
-            try:
-                for value in self[{"track_total": u"TRACKNUMBER",
-                                   "album_total": u"DISCNUMBER"}[attr]]:
-                    integer = re.search(r'/\D*(\d+)', value)
-                    if (integer is not None):
-                        return int(integer.group(1))
-            except KeyError:
-                # no slashed TRACKNUMBER/DISCNUMBER values either
-                # so return None
-                return None
-        elif (attr in self.ATTRIBUTE_MAP):
-            # attribute is supported by VorbisComment
-            try:
-                # if present, return the first value
-                return self[self.ATTRIBUTE_MAP[attr]][0]
-            except KeyError:
-                # if not present, return None
-                return None
-        elif (attr in self.FIELDS):
+            if attr in {'track_number', 'album_number'}:
+                try:
+                    # get the TRACKNUMBER/DISCNUMBER values
+                    # return the first value that contains an integer
+                    for value in self[key]:
+                        integer = re.search(r'\d+', value)
+                        if integer is not None:
+                            return int(integer.group(0))
+                    else:
+                        # otherwise, return None
+                        return None
+                except KeyError:
+                    # if no TRACKNUMBER/DISCNUMBER, return None
+                    return None
+            elif attr in {'track_total', 'album_total'}:
+                try:
+                    # get the TRACKTOTAL/DISCTOTAL values
+                    # return the first value that contains an integer
+                    for value in self[key]:
+                        integer = re.search(r'\d+', value)
+                        if integer is not None:
+                            return int(integer.group(0))
+                except KeyError:
+                    pass
+
+                # if no TRACKTOTAL/DISCTOTAL,
+                # or none of them contain an integer,
+                # look for slashed TRACKNUMBER/DISCNUMBER values
+                try:
+                    for value in self[{"track_total": u"TRACKNUMBER",
+                                       "album_total": u"DISCNUMBER"}[attr]]:
+                        if u"/" in value:
+                            integer = re.search(r'\d+',
+                                                value.split(u"/", 1)[1])
+                            if integer is not None:
+                                return int(integer.group(0))
+                    else:
+                        return None
+                except KeyError:
+                    # no slashed TRACKNUMBER/DISCNUMBER values either
+                    # so return None
+                    return None
+            elif attr == "compilation":
+                try:
+                    # if present, return True if the first value is "1"
+                    return self[key][0] == u"1"
+                except KeyError:
+                    # if not present, return None
+                    return None
+            else:
+                # attribute is supported by VorbisComment
+                try:
+                    # if present, return the first value
+                    return self[key][0]
+                except KeyError:
+                    # if not present, return None
+                    return None
+        elif attr in self.FIELDS:
             # attribute is supported by MetaData
             # but not supported by VorbisComment
             return None
@@ -238,94 +272,97 @@ class VorbisComment(MetaData):
         # updates the first matching field for the given attribute
         # in our list of comment strings
 
-        if ((value is None) and (attr in self.FIELDS)):
+        def has_number(unicode_string):
+            import re
+
+            return re.search(r'\d+', unicode_string) is not None
+
+        def swap_number(unicode_value, new_number):
+            import re
+
+            return re.sub(r'\d+', u"%d" % (new_number), unicode_value, 1)
+
+        if (attr in self.FIELDS) and (value is None):
             # setting any value to None is equivilent to deleting it
             # in this high-level implementation
             delattr(self, attr)
-        elif ((attr == "track_number") or (attr == "album_number")):
+        elif attr in self.ATTRIBUTE_MAP:
             key = self.ATTRIBUTE_MAP[attr]
-            try:
-                new_values = self[key]
-                for i in range(len(new_values)):
-                    # look for the first TRACKNUMBER/DISCNUMBER field
-                    # which contains an integer
-                    if (re.search(r'\d+', new_values[i]) is not None):
-                        # and replace the integer part of the field
-                        new_values[i] = re.sub(r'\d+',
-                                               unicode(int(value)),
-                                               new_values[i],
-                                               1)
 
-                        # then set the field to the new set of values
-                        # (which may contain subsequent fields to leave as-is)
-                        self[key] = new_values
-                        break
-                else:
-                    # no integer field with matching key
-                    # so append new integer field
-                    self[key] = self[key] + [unicode(int(value))]
-            except KeyError:
-                # no TRACKNUMBER/DISCNUMBER field
-                # so add a new one
-                self[key] = [unicode(int(value))]
-        elif ((attr == "track_total") or (attr == "album_total")):
-            key = self.ATTRIBUTE_MAP[attr]
-            try:
-                new_values = self[key]
-                for i in range(len(new_values)):
-                    # look for the first TRACKTOTAL/DISCTOTAL field
-                    # which contains an integer
-                    if (re.search(r'\d+', new_values[i]) is not None):
-                        # and replace the integer part of the field
-                        new_values[i] = re.sub(r'\d+',
-                                               unicode(int(value)),
-                                               new_values[i],
-                                               1)
-                        self[key] = new_values
-                        return
-            except KeyError:
-                new_values = []
+            if attr in {'track_number', 'album_number'}:
+                try:
+                    current_values = self[key]
+                    for i in range(len(current_values)):
+                        current_value = current_values[i]
+                        if u"/" not in current_value:
+                            if has_number(current_value):
+                                current_values[i] = swap_number(current_value,
+                                                                value)
+                                self[key] = current_values
+                                break
+                        else:
+                            (first, second) = current_value.split(u"/", 1)
+                            if has_number(first):
+                                current_values[i] = u"/".join(
+                                    [swap_number(first, value), second])
+                                self[key] = current_values
+                                break
+                    else:
+                        # no integer field matching key, so add new one
+                        self[key] = current_values + [u"%d" % (value)]
+                except KeyError:
+                    # no current field with key, so add new one
+                    self[key] = [u"%d" % (value)]
+            elif attr in {'track_total', 'album_total'}:
+                # look for standalone TRACKTOTAL/DISCTOTAL field
+                try:
+                    current_values = self[key]
 
-            # no TRACKTOTAL/DISCTOTAL field
-            # or none of them contain an integer,
-            # so look for slashed TRACKNUMBER/DISCNUMBER values
-            try:
-                slashed_key = {"track_total": u"TRACKNUMBER",
+                    for i in range(len(current_values)):
+                        current_value = current_values[i]
+                        if has_number(current_value):
+                            current_values[i] = swap_number(current_value,
+                                                            value)
+                            self[key] = current_values
+                            return
+                except KeyError:
+                    current_values = []
+
+                # no TRACKTOTAL/DISCTOTAL field
+                # or none of them contain an integer,
+                # so look for slashed TRACKNUMBER/DISCNUMBER values
+                try:
+                    new_key = {"track_total": u"TRACKNUMBER",
                                "album_total": u"DISCNUMBER"}[attr]
-                new_slashed_values = self[slashed_key]
-                for i in range(len(new_slashed_values)):
-                    # look for the first TRACKNUMBER/DISCNUMBER field
-                    # which contains a slashed value
-                    if (re.search(r'/\D*\d+',
-                                  new_slashed_values[i]) is not None):
-                        # and replace the slashed part of the field
-                        new_slashed_values[i] = re.sub(
-                            r'(/\D*)(\d+)',
-                            u'\\g<1>' + unicode(int(value)),
-                            new_slashed_values[i],
-                            1)
-                        self[slashed_key] = new_slashed_values
-                        return
-            except KeyError:
-                # no TRACKNUMBER/DISCNUMBER field found
-                pass
+                    slashed_values = self[new_key]
 
-            # no TRACKTOTAL/DISCTOTAL fields
-            # or no integer values in those fields,
-            # and no slashed TRACKNUMBER/DISCNUMBER values
-            # or no integer values in those fields,
-            # so append a TRACKTOTAL/DISCTOTAL field
-            self[key] = new_values + [unicode(int(value))]
-        elif (attr in self.ATTRIBUTE_MAP.keys()):
-            key = self.ATTRIBUTE_MAP[attr]
-            try:
-                current_values = self[key]
-                # try to leave subsequent fields with the same key as-is
-                self[key] = [unicode(value)] + current_values[1:]
-            except KeyError:
-                # no current field with the same key, so add a new one
-                self[key] = [unicode(value)]
-        elif (attr in self.FIELDS):
+                    for i in range(len(slashed_values)):
+                        current_value = slashed_values[i]
+                        if u"/" in current_value:
+                            (first, second) = current_value.split(u"/", 1)
+                            if has_number(second):
+                                slashed_values[i] = u"/".join(
+                                    [first, swap_number(second, value)])
+                                self[new_key] = slashed_values
+                                return
+                except KeyError:
+                    # no TRACKNUMBER/DISCNUMBER field found
+                    pass
+
+                # no slashed TRACKNUMBER/DISCNUMBER values either
+                # so append a TRACKTOTAL/DISCTOTAL field
+                self[key] = current_values + [u"%d" % (value)]
+            elif attr == "compilation":
+                self[key] = [u"1" if value else u"0"]
+            else:
+                # leave subsequent fields with the same key as-is
+                try:
+                    current_values = self[key]
+                    self[key] = [value] + current_values[1:]
+                except KeyError:
+                    # no current field with key, so add new one
+                    self[key] = [value]
+        elif attr in self.FIELDS:
             # attribute is supported by MetaData
             # but not supported by VorbisComment
             # so ignore it
@@ -335,63 +372,67 @@ class VorbisComment(MetaData):
             MetaData.__setattr__(self, attr, value)
 
     def __delattr__(self, attr):
+        #FIXME
         # deletes all matching keys for the given attribute
         # in our list of comment strings
 
-        if ((attr == "track_number") or (attr == "album_number")):
+        import re
+
+        if attr in self.ATTRIBUTE_MAP:
             key = self.ATTRIBUTE_MAP[attr]
-            try:
-                # convert the slashed side of TRACKNUMBER/DISCNUMBER fields
-                # to TRACKTOTAL/DISCKTOTAL fields
-                slashed_field = re.compile(r'/\s*(.*)')
 
-                orphaned_totals = [match.group(1) for match in
-                                   [slashed_field.search(value)
-                                    for value in self[key]]
-                                   if match is not None]
+            if attr in {'track_number', 'album_number'}:
+                try:
+                    current_values = self[key]
 
-                # remove any TRACKNUMBER/DISCNUMBER fields
+                    # save the _total side of any slashed fields for later
+                    slashed_totals = [int(match.group(0)) for match in
+                                      [re.search(r'\d+',
+                                                 value.split(u"/", 1)[1])
+                                       for value in current_values if
+                                       u"/" in value]
+                                      if match is not None]
+
+                    # remove the TRACKNUMBER/DISCNUMBER field itself
+                    self[key] = []
+
+                    # if there are any slashed totals
+                    # and there isn't a TRACKTOTAL/DISCTOTAL field already,
+                    # add a new one
+                    total_key = {'track_number': u"TRACKTOTAL",
+                                 'album_number': u"DISCTOTAL"}[attr]
+
+                    if (len(slashed_totals) > 0) and (total_key not in self):
+                        self[total_key] = [u"%d" % (slashed_totals[0])]
+                except KeyError:
+                    # no TRACKNUMBER/DISCNUMBER field to remove
+                    pass
+            elif attr in {'track_total', 'album_total'}:
+                def slash_filter(unicode_string):
+                    if u"/" not in unicode_string:
+                        return unicode_string
+                    else:
+                        return unicode_string.split(u"/", 1)[0].rstrip()
+
+                slashed_key = {"track_total": u"TRACKNUMBER",
+                               "album_total": u"DISCNUMBER"}[attr]
+
+                # remove TRACKTOTAL/DISCTOTAL fields
                 self[key] = []
 
-                if (len(orphaned_totals) > 0):
-                    total_key = {"track_number": u"TRACKTOTAL",
-                                 "album_number": u"DISCTOTAL"}[attr]
-                    try:
-                        # append new TRACKTOTAL/DISCTOTAL fields
-                        self[total_key] = self[total_key] + orphaned_totals
-                    except KeyError:
-                        # no TRACKTOTAL/DISCTOTAL field, so add new ones
-                        self[total_key] = orphaned_totals
-            except KeyError:
-                # no TRACKNUMBER/DISCNUMBER fields to remove
-                pass
-        elif ((attr == "track_total") or (attr == "album_total")):
-            slashed_key = {"track_total": u"TRACKNUMBER",
-                           "album_total": u"DISCNUMBER"}[attr]
-            slashed_field = re.compile(r'(.*?)\s*/.*')
-
-            def slash_filter(s):
-                match = slashed_field.match(s)
-                if (match is not None):
-                    return match.group(1)
-                else:
-                    return s
-
-            # remove TRACKTOTAL/DISCTOTAL fields
-            self[self.ATTRIBUTE_MAP[attr]] = []
-
-            # preserve the non-slashed side of TRACKNUMBER/DISCNUMBER fields
-            try:
-                self[slashed_key] = map(slash_filter, self[slashed_key])
-            except KeyError:
-                # no TRACKNUMBER/DISCNUMBER fields
-                pass
-        elif (attr in self.ATTRIBUTE_MAP):
-            # unlike __setattr_, which tries to preserve multiple instances
-            # of fields, __delattr__ wipes them all
-            # so that orphaned fields don't show up after deletion
-            self[self.ATTRIBUTE_MAP[attr]] = []
-        elif (attr in self.FIELDS):
+                # preserve the non-slashed side of TRACKNUMBER/DISCNUMBER fields
+                try:
+                    self[slashed_key] = [slash_filter(s) for s in
+                                         self[slashed_key]]
+                except KeyError:
+                    # no TRACKNUMBER/DISCNUMBER fields
+                    pass
+            else:
+                # unlike __setattr_, which tries to preserve multiple instances
+                # of fields, __delattr__ wipes them all
+                # so that orphaned fields don't show up after deletion
+                self[key] = []
+        elif attr in self.FIELDS:
             # attribute is part of MetaData
             # but not supported by VorbisComment
             pass
@@ -399,7 +440,7 @@ class VorbisComment(MetaData):
             MetaData.__delattr__(self, attr)
 
     def __eq__(self, metadata):
-        if (isinstance(metadata, self.__class__)):
+        if isinstance(metadata, self.__class__):
             return self.comment_strings == metadata.comment_strings
         else:
             return MetaData.__eq__(self, metadata)
@@ -410,13 +451,13 @@ class VorbisComment(MetaData):
 
         from audiotools import VERSION
 
-        if (metadata is None):
+        if metadata is None:
             return None
-        elif (isinstance(metadata, VorbisComment)):
+        elif isinstance(metadata, VorbisComment):
             return cls(metadata.comment_strings[:],
                        metadata.vendor_string)
-        elif (metadata.__class__.__name__ == 'FlacMetaData'):
-            if (metadata.has_block(4)):
+        elif metadata.__class__.__name__ == 'FlacMetaData':
+            if metadata.has_block(4):
                 vorbis_comment = metadata.get_block(4)
                 return cls(vorbis_comment.comment_strings[:],
                            vorbis_comment.vendor_string)
@@ -431,8 +472,17 @@ class VorbisComment(MetaData):
 
             for (attr, key) in cls.ATTRIBUTE_MAP.items():
                 value = getattr(metadata, attr)
-                if (value is not None):
-                    comment_strings.append(u"%s=%s" % (key, value))
+                if value is not None:
+                    attr_type = cls.FIELD_TYPES[attr]
+                    if attr_type is type(u""):
+                        comment_strings.append(
+                            u"%s=%s" % (key, value))
+                    elif attr_type is int:
+                        comment_strings.append(
+                            u"%s=%d" % (key, value))
+                    elif attr_type is bool:
+                        comment_strings.append(
+                            u"%s=%d" % (key, 1 if value else 0))
 
             return cls(comment_strings, u"Python Audio Tools %s" % (VERSION))
 
@@ -466,31 +516,31 @@ class VorbisComment(MetaData):
         reverse_attr_map = {}
         for (attr, key) in self.ATTRIBUTE_MAP.items():
             reverse_attr_map[key] = attr
-            if (key in self.ALIASES):
+            if key in self.ALIASES:
                 for alias in self.ALIASES[key]:
                     reverse_attr_map[alias] = attr
 
         cleaned_fields = []
 
         for comment_string in self.comment_strings:
-            if (u"=" in comment_string):
+            if u"=" in comment_string:
                 (key, value) = comment_string.split(u"=", 1)
-                if (key.upper() in reverse_attr_map):
+                if key.upper() in reverse_attr_map:
                     attr = reverse_attr_map[key.upper()]
                     # handle all text fields by stripping whitespace
-                    if (len(value.strip()) == 0):
+                    if len(value.strip()) == 0:
                         fixes_performed.append(
                             CLEAN_REMOVE_EMPTY_TAG %
                             {"field": key})
                     else:
                         fix1 = value.rstrip()
-                        if (fix1 != value):
+                        if fix1 != value:
                             fixes_performed.append(
                                 CLEAN_REMOVE_TRAILING_WHITESPACE %
                                 {"field": key})
 
                         fix2 = fix1.lstrip()
-                        if (fix2 != fix1):
+                        if fix2 != fix1:
                             fixes_performed.append(
                                 CLEAN_REMOVE_LEADING_WHITESPACE %
                                 {"field": key})
@@ -499,14 +549,14 @@ class VorbisComment(MetaData):
                         if (((attr == "track_number") or
                              (attr == "album_number"))):
                             match = re.match(r'(.*?)\s*/\s*(.*)', fix2)
-                            if (match is not None):
+                            if match is not None:
                                 # fix whitespace/zeroes
                                 # on either side of slash
                                 fix3 = u"%s/%s" % (
                                     match.group(1).lstrip(u"0"),
                                     match.group(2).lstrip(u"0"))
 
-                                if (fix3 != fix2):
+                                if fix3 != fix2:
                                     fixes_performed.append(
                                         CLEAN_REMOVE_LEADING_WHITESPACE_ZEROES
                                         % {"field": key})
@@ -514,14 +564,14 @@ class VorbisComment(MetaData):
                                 # fix zeroes only
                                 fix3 = fix2.lstrip(u"0")
 
-                                if (fix3 != fix2):
+                                if fix3 != fix2:
                                     fixes_performed.append(
                                         CLEAN_REMOVE_LEADING_ZEROES %
                                         {"field": key})
                         elif ((attr == "track_total") or
                               (attr == "album_total")):
                             fix3 = fix2.lstrip(u"0")
-                            if (fix3 != fix2):
+                            if fix3 != fix2:
                                 fixes_performed.append(
                                     CLEAN_REMOVE_LEADING_ZEROES %
                                     {"field": key})
@@ -536,3 +586,35 @@ class VorbisComment(MetaData):
 
         return (self.__class__(cleaned_fields, self.vendor_string),
                 fixes_performed)
+
+    def intersection(self, metadata):
+        """given a MetaData-compatible object,
+        returns a new MetaData object which contains
+        all the matching fields and images of this object and 'metadata'
+        """
+
+        def comment_present(comment):
+            if u"=" in comment:
+                key, value = comment.split(u"=", 1)
+                try:
+                    for other_value in metadata[key]:
+                        if value == other_value:
+                            return True
+                    else:
+                        return False
+                except KeyError:
+                    return False
+            else:
+                for other_comment in metadata.comment_strings:
+                    if comment == other_comment:
+                        return True
+                else:
+                    return False
+
+        if isinstance(metadata, VorbisComment):
+            return self.__class__([comment
+                                   for comment in self.comment_strings
+                                   if comment_present(comment)],
+                                  self.vendor_string)
+        else:
+            return MetaData.intersection(self, metadata)
