@@ -1,7 +1,5 @@
-#!/usr/bin/python
-
 # Audio Tools, a module and set of tools for manipulating audio data
-# Copyright (C) 2007-2014  Brian Langenberger
+# Copyright (C) 2007-2015  Brian Langenberger
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -20,6 +18,7 @@
 
 from audiotools import (AudioFile, InvalidFile, PCMReader, WaveContainer)
 from audiotools.pcm import FrameList
+import sys
 import struct
 
 
@@ -49,7 +48,7 @@ class RIFF_Chunk(object):
         """returns the total size of the chunk
         including the 8 byte ID/size and any padding byte"""
 
-        if (self.__size__ % 2):
+        if self.__size__ % 2:
             return 8 + self.__size__ + 1
         else:
             return 8 + self.__size__
@@ -74,8 +73,8 @@ class RIFF_Chunk(object):
         f.write(self.id)
         f.write(struct.pack("<I", self.__size__))
         f.write(self.__data__)
-        if (self.__size__ % 2):
-            f.write(chr(0))
+        if self.__size__ % 2:
+            f.write(b"\x00")
         return self.total_size()
 
 
@@ -95,6 +94,9 @@ class RIFF_File_Chunk(RIFF_Chunk):
         self.__wav_file__ = wav_file
         self.__offset__ = chunk_data_offset
 
+    def __del__(self):
+        self.__wav_file__.close()
+
     def __repr__(self):
         return "RIFF_File_Chunk(%s)" % (repr(self.id))
 
@@ -112,9 +114,9 @@ class RIFF_File_Chunk(RIFF_Chunk):
 
         self.__wav_file__.seek(self.__offset__)
         to_read = self.__size__
-        while (to_read > 0):
+        while to_read > 0:
             s = self.__wav_file__.read(min(0x100000, to_read))
-            if (len(s) == 0):
+            if len(s) == 0:
                 return False
             else:
                 to_read -= len(s)
@@ -129,13 +131,13 @@ class RIFF_File_Chunk(RIFF_Chunk):
         f.write(struct.pack("<I", self.__size__))
         self.__wav_file__.seek(self.__offset__)
         to_write = self.__size__
-        while (to_write > 0):
+        while to_write > 0:
             s = self.__wav_file__.read(min(0x100000, to_write))
             f.write(s)
             to_write -= len(s)
 
-        if (self.__size__ % 2):
-            f.write(chr(0))
+        if self.__size__ % 2:
+            f.write(b"\x00")
         return self.total_size()
 
 
@@ -164,14 +166,14 @@ def validate_header(header):
     from audiotools.bitstream import BitstreamReader
 
     header_size = len(header)
-    wave_file = BitstreamReader(BytesIO(header), 1)
+    wave_file = BitstreamReader(BytesIO(header), True)
     try:
         # ensure header starts with RIFF<size>WAVE chunk
         (riff, remaining_size, wave) = wave_file.parse("4b 32u 4b")
-        if (riff != "RIFF"):
+        if riff != b"RIFF":
             from audiotools.text import ERR_WAV_NOT_WAVE
             raise ValueError(ERR_WAV_NOT_WAVE)
-        elif (wave != "WAVE"):
+        elif wave != b"WAVE":
             from audiotools.text import ERR_WAV_INVALID_WAVE
             raise ValueError(ERR_WAV_INVALID_WAVE)
         else:
@@ -180,20 +182,20 @@ def validate_header(header):
 
         fmt_found = False
 
-        while (header_size > 0):
+        while header_size > 0:
             # ensure each chunk header is valid
             (chunk_id, chunk_size) = wave_file.parse("4b 32u")
-            if (not frozenset(chunk_id).issubset(WaveAudio.PRINTABLE_ASCII)):
+            if not frozenset(chunk_id).issubset(WaveAudio.PRINTABLE_ASCII):
                 from audiotools.text import ERR_WAV_INVALID_CHUNK
                 raise ValueError(ERR_WAV_INVALID_CHUNK)
             else:
                 header_size -= 8
 
-            if (chunk_id == "fmt "):
-                if (not fmt_found):
+            if chunk_id == b"fmt ":
+                if not fmt_found:
                     # skip fmt chunk data when found
                     fmt_found = True
-                    if (chunk_size % 2):
+                    if chunk_size % 2:
                         wave_file.skip_bytes(chunk_size + 1)
                         header_size -= (chunk_size + 1)
                     else:
@@ -203,12 +205,12 @@ def validate_header(header):
                     # ensure only one fmt chunk is found
                     from audiotools.text import ERR_WAV_MULTIPLE_FMT
                     raise ValueError(ERR_WAV_MULTIPLE_FMT)
-            elif (chunk_id == "data"):
-                if (not fmt_found):
+            elif chunk_id == b"data":
+                if not fmt_found:
                     # ensure at least one fmt chunk is found
                     from audiotools.text import ERR_WAV_PREMATURE_DATA
                     raise ValueError(ERR_WAV_PREMATURE_DATA)
-                elif (header_size > 0):
+                elif header_size > 0:
                     # ensure no data remains after data chunk header
                     from audiotools.text import ERR_WAV_HEADER_EXTRA_DATA
                     raise ValueError(ERR_WAV_HEADER_EXTRA_DATA %
@@ -217,7 +219,7 @@ def validate_header(header):
                     return (total_size, chunk_size)
             else:
                 # skip the full contents of non-audio chunks
-                if (chunk_size % 2):
+                if chunk_size % 2:
                     wave_file.skip_bytes(chunk_size + 1)
                     header_size -= (chunk_size + 1)
                 else:
@@ -242,33 +244,33 @@ def validate_footer(footer, data_bytes_written):
     from audiotools.bitstream import BitstreamReader
 
     total_size = len(footer)
-    wave_file = BitstreamReader(BytesIO(footer), 1)
+    wave_file = BitstreamReader(BytesIO(footer), True)
     try:
         # ensure footer is padded properly if necessary
         # based on size of data bytes written
-        if (data_bytes_written % 2):
+        if data_bytes_written % 2:
             wave_file.skip_bytes(1)
             total_size -= 1
 
-        while (total_size > 0):
+        while total_size > 0:
             (chunk_id, chunk_size) = wave_file.parse("4b 32u")
-            if (not frozenset(chunk_id).issubset(WaveAudio.PRINTABLE_ASCII)):
+            if not frozenset(chunk_id).issubset(WaveAudio.PRINTABLE_ASCII):
                 from audiotools.text import ERR_WAV_INVALID_CHUNK
                 raise ValueError(ERR_WAV_INVALID_CHUNK)
             else:
                 total_size -= 8
 
-            if (chunk_id == "fmt "):
+            if chunk_id == b"fmt ":
                 # ensure no fmt chunks are found
                 from audiotools.text import ERR_WAV_MULTIPLE_FMT
                 raise ValueError(ERR_WAV_MULTIPLE_FMT)
-            elif (chunk_id == "data"):
+            elif chunk_id == b"data":
                 # ensure no data chunks are found
                 from audiotools.text import ERR_WAV_MULTIPLE_DATA
                 raise ValueError(ERR_WAV_MULTIPLE_DATA)
             else:
                 # skip the full contents of non-audio chunks
-                if (chunk_size % 2):
+                if chunk_size % 2:
                     wave_file.skip_bytes(chunk_size + 1)
                     total_size -= (chunk_size + 1)
                 else:
@@ -297,32 +299,32 @@ def parse_fmt(fmt):
      block_align,
      bits_per_sample) = fmt.parse("16u 16u 32u 32u 16u 16u")
 
-    if (compression == 1):
+    if compression == 1:
         # if we have a multi-channel WAVE file
         # that's not WAVEFORMATEXTENSIBLE,
         # assume the channels follow
         # SMPTE/ITU-R recommendations
         # and hope for the best
-        if (channels == 1):
+        if channels == 1:
             channel_mask = ChannelMask.from_fields(
                 front_center=True)
-        elif (channels == 2):
+        elif channels == 2:
             channel_mask = ChannelMask.from_fields(
                 front_left=True, front_right=True)
-        elif (channels == 3):
+        elif channels == 3:
             channel_mask = ChannelMask.from_fields(
                 front_left=True, front_right=True,
                 front_center=True)
-        elif (channels == 4):
+        elif channels == 4:
             channel_mask = ChannelMask.from_fields(
                 front_left=True, front_right=True,
                 back_left=True, back_right=True)
-        elif (channels == 5):
+        elif channels == 5:
             channel_mask = ChannelMask.from_fields(
                 front_left=True, front_right=True,
                 back_left=True, back_right=True,
                 front_center=True)
-        elif (channels == 6):
+        elif channels == 6:
             channel_mask = ChannelMask.from_fields(
                 front_left=True, front_right=True,
                 back_left=True, back_right=True,
@@ -331,14 +333,14 @@ def parse_fmt(fmt):
             channel_mask = ChannelMask(0)
 
         return (channels, sample_rate, bits_per_sample, channel_mask)
-    elif (compression == 0xFFFE):
+    elif compression == 0xFFFE:
         (cb_size,
          valid_bits_per_sample,
          channel_mask,
          sub_format) = fmt.parse("16u 16u 32u 16b")
         if (sub_format !=
-            ('\x01\x00\x00\x00\x00\x00\x10\x00' +
-             '\x80\x00\x00\xaa\x00\x38\x9b\x71')):
+            (b'\x01\x00\x00\x00\x00\x00\x10\x00' +
+             b'\x80\x00\x00\xaa\x00\x38\x9b\x71')):
             # FIXME
             raise ValueError("invalid WAVE sub-format")
         else:
@@ -362,6 +364,13 @@ def wave_header(sample_rate,
 
     from audiotools.bitstream import (BitstreamRecorder, format_size)
 
+    assert(isinstance(sample_rate, int))
+    assert(isinstance(channels, int))
+    assert(isinstance(channel_mask, int))
+    assert(isinstance(bits_per_sample, int))
+    assert(isinstance(total_pcm_frames, int) or
+           isinstance(total_pcm_frames, long))
+
     header = BitstreamRecorder(True)
 
     avg_bytes_per_second = sample_rate * channels * (bits_per_sample // 8)
@@ -369,7 +378,7 @@ def wave_header(sample_rate,
 
     # build a regular or extended fmt chunk
     # based on the reader's attributes
-    if (((channels <= 2) and (bits_per_sample <= 16))):
+    if ((channels <= 2) and (bits_per_sample <= 16)):
         fmt = "16u 16u 32u 32u 16u 16u"
         fmt_fields = (1,   # compression code
                       channels,
@@ -378,7 +387,7 @@ def wave_header(sample_rate,
                       block_align,
                       bits_per_sample)
     else:
-        if (channel_mask == 0):
+        if channel_mask == 0:
             channel_mask = {1: 0x4,
                             2: 0x3,
                             3: 0x7,
@@ -394,20 +403,20 @@ def wave_header(sample_rate,
                       bits_per_sample,
                       22,       # CB size
                       bits_per_sample,
-                      int(channel_mask),
-                      '\x01\x00\x00\x00\x00\x00\x10\x00' +
-                      '\x80\x00\x00\xaa\x00\x38\x9b\x71'  # sub format
+                      channel_mask,
+                      b'\x01\x00\x00\x00\x00\x00\x10\x00' +
+                      b'\x80\x00\x00\xaa\x00\x38\x9b\x71'  # sub format
                       )
 
     data_size = (bits_per_sample // 8) * channels * total_pcm_frames
     total_size = ((format_size("4b" + "4b 32u" + fmt + "4b 32u") // 8) +
                   data_size + (data_size % 2))
 
-    if (total_size < (2 ** 32)):
-        header.build("4b 32u 4b", ("RIFF", total_size, "WAVE"))
-        header.build("4b 32u", ("fmt ", format_size(fmt) // 8))
+    if total_size < (2 ** 32):
+        header.build("4b 32u 4b", (b"RIFF", total_size, b"WAVE"))
+        header.build("4b 32u", (b"fmt ", format_size(fmt) // 8))
         header.build(fmt, fmt_fields)
-        header.build("4b 32u", ("data", data_size))
+        header.build("4b 32u", (b"data", data_size))
 
         return header.data()
     else:
@@ -422,72 +431,79 @@ class WaveReader(object):
 
         from audiotools.bitstream import BitstreamReader
 
-        self.file = open(wave_filename, "rb")
+        self.stream = BitstreamReader(open(wave_filename, "rb"), True)
 
         # ensure RIFF<size>WAVE header is ok
         try:
             (riff,
              total_size,
-             wave) = struct.unpack("<4sI4s", self.file.read(12))
+             wave) = self.stream.parse("4b 32u 4b")
         except struct.error:
             from audiotools.text import ERR_WAV_INVALID_WAVE
+            self.stream.close()
             raise ValueError(ERR_WAV_INVALID_WAVE)
 
-        if (riff != 'RIFF'):
+        if riff != b'RIFF':
             from audiotools.text import ERR_WAV_NOT_WAVE
+            self.stream.close()
             raise ValueError(ERR_WAV_NOT_WAVE)
-        elif (wave != 'WAVE'):
+        elif wave != b'WAVE':
             from audiotools.text import ERR_WAV_INVALID_WAVE
+            self.stream.close()
             raise ValueError(ERR_WAV_INVALID_WAVE)
         else:
             total_size -= 4
             fmt_chunk_read = False
 
         # walk through chunks until "data" chunk encountered
-        while (total_size > 0):
+        while total_size > 0:
             try:
                 (chunk_id,
-                 chunk_size) = struct.unpack("<4sI", self.file.read(8))
+                 chunk_size) = self.stream.parse("4b 32u")
             except struct.error:
                 from audiotools.text import ERR_WAV_INVALID_WAVE
+                self.stream.close()
                 raise ValueError(ERR_WAV_INVALID_WAVE)
-            if (not frozenset(chunk_id).issubset(WaveAudio.PRINTABLE_ASCII)):
+            if not frozenset(chunk_id).issubset(WaveAudio.PRINTABLE_ASCII):
                 from audiotools.text import ERR_WAV_INVALID_CHUNK
+                self.stream.close()
                 raise ValueError(ERR_WAV_INVALID_CHUNK)
             else:
                 total_size -= 8
 
-            if (chunk_id == "fmt "):
+            if chunk_id == b"fmt ":
                 # when "fmt " chunk encountered,
                 # use it to populate PCMReader attributes
                 (self.channels,
                  self.sample_rate,
                  self.bits_per_sample,
-                 channel_mask) = parse_fmt(BitstreamReader(self.file, True))
+                 channel_mask) = parse_fmt(self.stream)
                 self.channel_mask = int(channel_mask)
                 self.bytes_per_pcm_frame = ((self.bits_per_sample // 8) *
                                             self.channels)
                 fmt_chunk_read = True
-            elif (chunk_id == "data"):
+            elif chunk_id == b"data":
                 # when "data" chunk encountered,
                 # use its size to determine total PCM frames
                 # and ready PCMReader for reading
-                if (not fmt_chunk_read):
+                if not fmt_chunk_read:
                     from audiotools.text import ERR_WAV_PREMATURE_DATA
+                    self.stream.close()
                     raise ValueError(ERR_WAV_PREMATURE_DATA)
                 else:
                     self.total_pcm_frames = (chunk_size //
                                              self.bytes_per_pcm_frame)
                     self.remaining_pcm_frames = self.total_pcm_frames
-                    self.data_chunk_offset = self.file.tell()
+                    self.data_start = self.stream.getpos()
                     return
             else:
                 # all other chunks are ignored
-                self.file.read(chunk_size)
+                self.stream.skip_bytes(chunk_size)
 
-            if (chunk_size % 2):
-                if (len(self.file.read(1)) < 1):
+            if chunk_size % 2:
+                if len(self.stream.read_bytes(1)) < 1:
                     from audiotools.text import ERR_WAV_INVALID_CHUNK
+                    self.stream.close()
                     raise ValueError(ERR_WAV_INVALID_CHUNK)
                 total_size -= (chunk_size + 1)
             else:
@@ -495,8 +511,15 @@ class WaveReader(object):
         else:
             # raise an error if no "data" chunk is encountered
             from audiotools.text import ERR_WAV_NO_DATA_CHUNK
+            self.stream.close()
             raise ValueError(ERR_WAV_NO_DATA_CHUNK)
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
     def read(self, pcm_frames):
         """try to read a pcm.FrameList with the given number of PCM frames"""
 
@@ -506,10 +529,10 @@ class WaveReader(object):
 
         requested_bytes = (self.bytes_per_pcm_frame *
                            requested_pcm_frames)
-        pcm_data = self.file.read(requested_bytes)
+        pcm_data = self.stream.read_bytes(requested_bytes)
 
         # raise exception if "data" chunk exhausted early
-        if (len(pcm_data) < requested_bytes):
+        if len(pcm_data) < requested_bytes:
             from audiotools.text import ERR_WAV_TRUNCATED_DATA_CHUNK
             raise IOError(ERR_WAV_TRUNCATED_DATA_CHUNK)
         else:
@@ -522,31 +545,37 @@ class WaveReader(object):
                              False,
                              self.bits_per_sample != 8)
 
+    def read_closed(self, pcm_frames):
+        raise ValueError("cannot read closed stream")
+
     def seek(self, pcm_frame_offset):
         """tries to seek to the given PCM frame offset
         returns the total amount of frames actually seeked over"""
 
-        if (pcm_frame_offset < 0):
+        if pcm_frame_offset < 0:
             from audiotools.text import ERR_NEGATIVE_SEEK
             raise ValueError(ERR_NEGATIVE_SEEK)
 
         # ensure one doesn't walk off the end of the file
-        pcm_frame_offset = min(pcm_frame_offset,
-                               self.total_pcm_frames)
+        pcm_frame_offset = min(pcm_frame_offset, self.total_pcm_frames)
 
         # position file in "data" chunk
-        self.file.seek(self.data_chunk_offset +
-                       (pcm_frame_offset *
-                        self.bytes_per_pcm_frame), 0)
+        self.stream.setpos(self.data_start)
+        self.stream.seek(pcm_frame_offset * self.bytes_per_pcm_frame, 1)
         self.remaining_pcm_frames = (self.total_pcm_frames -
                                      pcm_frame_offset)
 
         return pcm_frame_offset
 
+    def seek_closed(self, pcm_frame_offset):
+        raise ValueError("cannot seek closed stream")
+
     def close(self):
         """closes the stream for reading"""
 
-        self.file.close()
+        self.stream.close()
+        self.read = self.read_closed
+        self.seek = self.seek_closed
 
 
 class TempWaveReader(WaveReader):
@@ -561,6 +590,7 @@ class TempWaveReader(WaveReader):
         self.tempfile = tempfile
 
     def __del__(self):
+        WaveReader.__del__(self)
         self.tempfile.close()
 
     def close(self):
@@ -582,7 +612,10 @@ class WaveAudio(WaveContainer):
     NAME = SUFFIX
     DESCRIPTION = u"Waveform Audio File Format"
 
-    PRINTABLE_ASCII = frozenset([chr(i) for i in range(0x20, 0x7E + 1)])
+    if sys.version_info[0] >= 3:
+        PRINTABLE_ASCII = {i for i in range(0x20, 0x7E + 1)}
+    else:
+        PRINTABLE_ASCII = {chr(i) for i in range(0x20, 0x7E + 1)}
 
     def __init__(self, filename):
         """filename is a plain string"""
@@ -603,27 +636,26 @@ class WaveAudio(WaveContainer):
 
         try:
             for chunk in self.chunks():
-                if (chunk.id == "fmt "):
+                if chunk.id == b"fmt ":
                     try:
                         (self.__channels__,
                          self.__sample_rate__,
                          self.__bits_per_sample__,
                          self.__channel_mask__) = parse_fmt(
-                            BitstreamReader(chunk.data(), 1))
+                            BitstreamReader(chunk.data(), True))
                         fmt_read = True
-                        if (fmt_read and data_read):
+                        if fmt_read and data_read:
                             break
                     except IOError:
                         continue
                     except ValueError as err:
                         raise InvalidWave(str(err))
-                elif (chunk.id == "data"):
+                elif chunk.id == b"data":
                     self.__data_size__ = chunk.size()
                     data_read = True
-                    if (fmt_read and data_read):
+                    if fmt_read and data_read:
                         break
         except IOError:
-            # FIXME
             raise InvalidWave("I/O error reading wave")
 
     def lossless(self):
@@ -639,18 +671,41 @@ class WaveAudio(WaveContainer):
         conversion should be routed through .wav conversion
         to avoid losing those chunks"""
 
-        return {'fmt ', 'data'} != {c.id for c in self.chunks()}
+        return {b'fmt ', b'data'} != {c.id for c in self.chunks()}
 
     def channel_mask(self):
         """returns a ChannelMask object of this track's channel layout"""
 
         return self.__channel_mask__
 
+    @classmethod
+    def supports_to_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .to_pcm() method"""
+
+        return True
+
     # Returns the PCMReader object for this WAV's data
     def to_pcm(self):
         """returns a PCMReader object containing the track's PCM data"""
 
-        return WaveReader(self.filename)
+        try:
+            return WaveReader(self.filename)
+        except (IOError, ValueError) as err:
+            from audiotools import PCMReaderError
+
+            return PCMReaderError(error_message=str(err),
+                                  sample_rate=self.sample_rate(),
+                                  channels=self.channels(),
+                                  channel_mask=int(self.channel_mask()),
+                                  bits_per_sample=self.bits_per_sample())
+
+    @classmethod
+    def supports_from_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .from_pcm() classmethod"""
+
+        return True
 
     @classmethod
     def from_pcm(cls, filename, pcmreader,
@@ -677,11 +732,13 @@ class WaveAudio(WaveContainer):
                                  total_pcm_frames if total_pcm_frames
                                  is not None else 0)
         except ValueError as err:
+            pcmreader.close()
             raise EncodingError(str(err))
 
         try:
             f = open(filename, "wb")
         except IOError as err:
+            pcmreader.close()
             raise EncodingError(str(err))
 
         counter = CounterPCMReader(pcmreader)
@@ -692,25 +749,26 @@ class WaveAudio(WaveContainer):
                                     pcmreader.bits_per_sample > 8,
                                     False)
         except (IOError, ValueError) as err:
+            f.close()
             cls.__unlink__(filename)
             raise EncodingError(str(err))
         except Exception as err:
+            f.close()
             cls.__unlink__(filename)
             raise err
 
         # handle odd-sized "data" chunks
-        if (counter.frames_written % 2):
-            f.write(chr(0))
+        if counter.frames_written % 2:
+            f.write(b"\x00")
 
-        if (total_pcm_frames is not None):
-            if (total_pcm_frames != counter.frames_written):
+        if total_pcm_frames is not None:
+            f.close()
+            if total_pcm_frames != counter.frames_written:
                 # ensure written number of PCM frames
                 # matches total_pcm_frames argument
                 from audiotools.text import ERR_TOTAL_PCM_FRAMES_MISMATCH
                 cls.__unlink__(filename)
                 raise EncodingError(ERR_TOTAL_PCM_FRAMES_MISMATCH)
-            else:
-                f.close()
         else:
             # go back and rewrite populated header
             # with counted number of PCM frames
@@ -764,7 +822,7 @@ class WaveAudio(WaveContainer):
         raises UnsupportedTracknameField if the format string
         contains invalid template fields"""
 
-        if (format is None):
+        if format is None:
             format = "track%(track_number)2.2d.wav"
         return AudioFile.track_name(file_path, track_metadata, format,
                                     suffix=cls.SUFFIX)
@@ -772,58 +830,58 @@ class WaveAudio(WaveContainer):
     def chunks(self):
         """yields a set of RIFF_Chunk or RIFF_File_Chunk objects"""
 
-        wave_file = open(self.filename, "rb")
-        try:
-            (riff,
-             total_size,
-             wave) = struct.unpack("<4sI4s", wave_file.read(12))
-        except struct.error:
-            from audiotools.text import ERR_WAV_INVALID_WAVE
-            raise InvalidWave(ERR_WAV_INVALID_WAVE)
-
-        if (riff != 'RIFF'):
-            from audiotools.text import ERR_WAV_NOT_WAVE
-            raise InvalidWave(ERR_WAV_NOT_WAVE)
-        elif (wave != 'WAVE'):
-            from audiotools.text import ERR_WAV_INVALID_WAVE
-            raise InvalidWave(ERR_WAV_INVALID_WAVE)
-        else:
-            total_size -= 4
-
-        while (total_size > 0):
-            # read the chunk header and ensure its validity
+        with open(self.filename, "rb") as wave_file:
             try:
-                (chunk_id,
-                 chunk_size) = struct.unpack("<4sI", wave_file.read(8))
+                (riff,
+                 total_size,
+                 wave) = struct.unpack("<4sI4s", wave_file.read(12))
             except struct.error:
                 from audiotools.text import ERR_WAV_INVALID_WAVE
                 raise InvalidWave(ERR_WAV_INVALID_WAVE)
-            if (not frozenset(chunk_id).issubset(self.PRINTABLE_ASCII)):
-                from audiotools.text import ERR_WAV_INVALID_CHUNK
-                raise InvalidWave(ERR_WAV_INVALID_CHUNK)
-            else:
-                total_size -= 8
 
-            # yield RIFF_Chunk or RIFF_File_Chunk depending on chunk size
-            if (chunk_size >= 0x100000):
-                # if chunk is too large, yield a File_Chunk
-                yield RIFF_File_Chunk(chunk_id,
-                                      chunk_size,
-                                      open(self.filename, "rb"),
-                                      wave_file.tell())
-                wave_file.seek(chunk_size, 1)
+            if riff != b'RIFF':
+                from audiotools.text import ERR_WAV_NOT_WAVE
+                raise InvalidWave(ERR_WAV_NOT_WAVE)
+            elif wave != b'WAVE':
+                from audiotools.text import ERR_WAV_INVALID_WAVE
+                raise InvalidWave(ERR_WAV_INVALID_WAVE)
             else:
-                # otherwise, yield a raw data Chunk
-                yield RIFF_Chunk(chunk_id, chunk_size,
-                                 wave_file.read(chunk_size))
+                total_size -= 4
 
-            if (chunk_size % 2):
-                if (len(wave_file.read(1)) < 1):
+            while total_size > 0:
+                # read the chunk header and ensure its validity
+                try:
+                    (chunk_id,
+                     chunk_size) = struct.unpack("<4sI", wave_file.read(8))
+                except struct.error:
+                    from audiotools.text import ERR_WAV_INVALID_WAVE
+                    raise InvalidWave(ERR_WAV_INVALID_WAVE)
+                if not frozenset(chunk_id).issubset(self.PRINTABLE_ASCII):
                     from audiotools.text import ERR_WAV_INVALID_CHUNK
                     raise InvalidWave(ERR_WAV_INVALID_CHUNK)
-                total_size -= (chunk_size + 1)
-            else:
-                total_size -= chunk_size
+                else:
+                    total_size -= 8
+
+                # yield RIFF_Chunk or RIFF_File_Chunk depending on chunk size
+                if chunk_size >= 0x100000:
+                    # if chunk is too large, yield a File_Chunk
+                    yield RIFF_File_Chunk(chunk_id,
+                                          chunk_size,
+                                          open(self.filename, "rb"),
+                                          wave_file.tell())
+                    wave_file.seek(chunk_size, 1)
+                else:
+                    # otherwise, yield a raw data Chunk
+                    yield RIFF_Chunk(chunk_id, chunk_size,
+                                     wave_file.read(chunk_size))
+
+                if chunk_size % 2:
+                    if len(wave_file.read(1)) < 1:
+                        from audiotools.text import ERR_WAV_INVALID_CHUNK
+                        raise InvalidWave(ERR_WAV_INVALID_CHUNK)
+                    total_size -= (chunk_size + 1)
+                else:
+                    total_size -= chunk_size
 
     @classmethod
     def wave_from_chunks(cls, filename, chunk_iter):
@@ -833,12 +891,12 @@ class WaveAudio(WaveContainer):
         chunk_iter should yield RIFF_Chunk-compatible objects
         """
 
-        wave_file = open(filename, 'wb')
-        try:
+        with open(filename, 'wb') as wave_file:
             total_size = 4
 
             # write an unfinished header with a placeholder size
-            wave_file.write(struct.pack("<4sI4s", "RIFF", total_size, "WAVE"))
+            wave_file.write(
+                struct.pack("<4sI4s", b"RIFF", total_size, b"WAVE"))
 
             # write the individual chunks
             for chunk in chunk_iter:
@@ -846,9 +904,8 @@ class WaveAudio(WaveContainer):
 
             # once the chunks are done, go back and re-write the header
             wave_file.seek(0, 0)
-            wave_file.write(struct.pack("<4sI4s", "RIFF", total_size, "WAVE"))
-        finally:
-            wave_file.close()
+            wave_file.write(
+                struct.pack("<4sI4s", b"RIFF", total_size, b"WAVE"))
 
     def wave_header_footer(self):
         """returns a pair of data strings before and after PCM data
@@ -876,24 +933,23 @@ class WaveAudio(WaveContainer):
         current_block = head
         fmt_found = False
 
-        wave_file = BitstreamReader(open(self.filename, 'rb'), 1)
-        try:
+        with BitstreamReader(open(self.filename, 'rb'), 1) as wave_file:
             # transfer the 12-byte "RIFFsizeWAVE" header to head
             (riff, size, wave) = wave_file.parse("4b 32u 4b")
-            if (riff != 'RIFF'):
+            if riff != b'RIFF':
                 from audiotools.text import ERR_WAV_NOT_WAVE
                 raise ValueError(ERR_WAV_NOT_WAVE)
-            elif (wave != 'WAVE'):
+            elif wave != b'WAVE':
                 from audiotools.text import ERR_WAV_INVALID_WAVE
                 raise ValueError(ERR_WAV_INVALID_WAVE)
             else:
                 current_block.build("4b 32u 4b", (riff, size, wave))
                 total_size = size - 4
 
-            while (total_size > 0):
+            while total_size > 0:
                 # transfer each chunk header
                 (chunk_id, chunk_size) = wave_file.parse("4b 32u")
-                if (not frozenset(chunk_id).issubset(self.PRINTABLE_ASCII)):
+                if not frozenset(chunk_id).issubset(self.PRINTABLE_ASCII):
                     from audiotools.text import ERR_WAV_INVALID_CHUNK
                     raise ValueError(ERR_WAV_INVALID_CHUNK)
                 else:
@@ -901,15 +957,15 @@ class WaveAudio(WaveContainer):
                     total_size -= 8
 
                 # and transfer the full content of non-audio chunks
-                if (chunk_id != "data"):
-                    if (chunk_id == "fmt "):
-                        if (not fmt_found):
+                if chunk_id != b"data":
+                    if chunk_id == b"fmt ":
+                        if not fmt_found:
                             fmt_found = True
                         else:
                             from audiotools.text import ERR_WAV_MULTIPLE_FMT
                             raise ValueError(ERR_WAV_MULTIPLE_FMT)
 
-                    if (chunk_size % 2):
+                    if chunk_size % 2:
                         current_block.write_bytes(
                             wave_file.read_bytes(chunk_size + 1))
                         total_size -= (chunk_size + 1)
@@ -921,19 +977,17 @@ class WaveAudio(WaveContainer):
                     wave_file.skip_bytes(chunk_size)
                     current_block = tail
 
-                    if (chunk_size % 2):
+                    if chunk_size % 2:
                         current_block.write_bytes(wave_file.read_bytes(1))
                         total_size -= (chunk_size + 1)
                     else:
                         total_size -= chunk_size
 
-            if (fmt_found):
+            if fmt_found:
                 return (head.data(), tail.data())
             else:
                 from audiotools.text import ERR_WAV_NO_FMT_CHUNK
-                return ValueError(ERR_WAV_NO_FMT_CHUNK)
-        finally:
-            wave_file.close()
+                raise ValueError(ERR_WAV_NO_FMT_CHUNK)
 
     @classmethod
     def from_wave(cls, filename, header, pcmreader, footer, compression=None):
@@ -960,52 +1014,53 @@ class WaveAudio(WaveContainer):
         try:
             (total_size, data_size) = validate_header(header)
         except ValueError as err:
+            pcmreader.close()
             raise EncodingError(str(err))
 
         try:
-            # write header to output file
             f = open(filename, "wb")
+        except IOError as msg:
+            pcmreader.close()
+            raise EncodingError(err)
+
+        try:
+            # write header to output file
             f.write(header)
 
             # write PCM data to output file
             data_bytes_written = 0
             signed = (pcmreader.bits_per_sample > 8)
             s = pcmreader.read(FRAMELIST_SIZE).to_bytes(False, signed)
-            while (len(s) > 0):
+            while len(s) > 0:
                 data_bytes_written += len(s)
                 f.write(s)
                 s = pcmreader.read(FRAMELIST_SIZE).to_bytes(False, signed)
 
             # ensure output data size matches the "data" chunk's size
-            if (data_size != data_bytes_written):
+            if data_size != data_bytes_written:
                 cls.__unlink__(filename)
                 from audiotools.text import ERR_WAV_TRUNCATED_DATA_CHUNK
                 raise EncodingError(ERR_WAV_TRUNCATED_DATA_CHUNK)
 
             # ensure footer validates correctly
-            try:
-                validate_footer(footer, data_bytes_written)
-                # before writing it to disk
-                f.write(footer)
-            except ValueError as err:
-                cls.__unlink__(filename)
-                raise EncodingError(str(err))
-
-            f.close()
+            # before writing it to disk
+            validate_footer(footer, data_bytes_written)
+            f.write(footer)
+            f.flush()
 
             # ensure total size is correct
-            if ((len(header) + data_size + len(footer)) != total_size):
+            if (len(header) + data_size + len(footer)) != total_size:
                 cls.__unlink__(filename)
                 from audiotools.text import ERR_WAV_INVALID_SIZE
                 raise EncodingError(ERR_WAV_INVALID_SIZE)
 
             return cls(filename)
-        except IOError as err:
+        except (IOError, ValueError) as err:
             cls.__unlink__(filename)
             raise EncodingError(str(err))
-        except DecodingError as err:
-            cls.__unlink__(filename)
-            raise EncodingError(err.error_message)
+        finally:
+            f.close()
+            pcmreader.close()
 
     def verify(self, progress=None):
         """verifies the current file for correctness
@@ -1021,28 +1076,28 @@ class WaveAudio(WaveContainer):
         try:
             (header, footer) = self.wave_header_footer()
         except IOError as err:
-            raise InvalidWave(unicode(err))
+            raise InvalidWave(err)
         except ValueError as err:
-            raise InvalidWave(unicode(err))
+            raise InvalidWave(err)
 
         # ensure header is valid
         try:
             (total_size, data_size) = validate_header(header)
         except ValueError as err:
-            raise InvalidWave(unicode(err))
+            raise InvalidWave(err)
 
         # ensure "data" chunk has all its data
         counter = CounterPCMReader(to_pcm_progress(self, progress))
         try:
             transfer_framelist_data(counter, lambda f: f)
-        except IOError:
+        except (IOError, ValueError):
             from audiotools.text import ERR_WAV_TRUNCATED_DATA_CHUNK
             raise InvalidWave(ERR_WAV_TRUNCATED_DATA_CHUNK)
 
         data_bytes_written = counter.bytes_written()
 
         # ensure output data size matches the "data" chunk's size
-        if (data_size != data_bytes_written):
+        if data_size != data_bytes_written:
             from audiotools.text import ERR_WAV_TRUNCATED_DATA_CHUNK
             raise InvalidWave(ERR_WAV_TRUNCATED_DATA_CHUNK)
 
@@ -1054,7 +1109,7 @@ class WaveAudio(WaveContainer):
             raise InvalidWave(ERR_WAV_INVALID_SIZE)
 
         # ensure total size is correct
-        if ((len(header) + data_size + len(footer)) != total_size):
+        if (len(header) + data_size + len(footer)) != total_size:
             from audiotools.text import ERR_WAV_INVALID_SIZE
             raise InvalidWave(ERR_WAV_INVALID_SIZE)
 
@@ -1078,21 +1133,21 @@ class WaveAudio(WaveContainer):
         pending_data = None
 
         for chunk in self.chunks():
-            if (chunk.id == "fmt "):
-                if ("fmt " in [c.id for c in chunk_queue]):
+            if chunk.id == b"fmt ":
+                if b"fmt " in [c.id for c in chunk_queue]:
                     from audiotools.text import CLEAN_WAV_MULTIPLE_FMT_CHUNKS
                     fixes_performed.append(CLEAN_WAV_MULTIPLE_FMT_CHUNKS)
                 else:
                     chunk_queue.append(chunk)
-                    if (pending_data is not None):
+                    if pending_data is not None:
                         chunk_queue.append(pending_data)
                         pending_data = None
-            elif (chunk.id == "data"):
-                if ("fmt " not in [c.id for c in chunk_queue]):
+            elif chunk.id == b"data":
+                if b"fmt " not in [c.id for c in chunk_queue]:
                     from audiotools.text import CLEAN_WAV_REORDERED_DATA_CHUNK
                     fixes_performed.append(CLEAN_WAV_REORDERED_DATA_CHUNK)
                     pending_data = chunk
-                elif ("data" in [c.id for c in chunk_queue]):
+                elif b"data" in [c.id for c in chunk_queue]:
                     from audiotools.text import CLEAN_WAV_MULTIPLE_DATA_CHUNKS
                     fixes_performed.append(CLEAN_WAV_MULTIPLE_DATA_CHUNKS)
                 else:
@@ -1100,7 +1155,7 @@ class WaveAudio(WaveContainer):
             else:
                 chunk_queue.append(chunk)
 
-        if (output_filename is not None):
+        if output_filename is not None:
             WaveAudio.wave_from_chunks(output_filename, chunk_queue)
 
         return fixes_performed
