@@ -1,7 +1,5 @@
-#!/usr/bin/python
-
 # Audio Tools, a module and set of tools for manipulating audio data
-# Copyright (C) 2007-2014  Brian Langenberger
+# Copyright (C) 2007-2015  Brian Langenberger
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -28,20 +26,20 @@ class InvalidWavPack(InvalidFile):
 
 def __riff_chunk_ids__(data_size, data):
     (riff, size, wave) = data.parse("4b 32u 4b")
-    if (riff != "RIFF"):
+    if riff != b"RIFF":
         return
-    elif (wave != "WAVE"):
+    elif wave != b"WAVE":
         return
     else:
         data_size -= 12
 
-    while (data_size > 0):
+    while data_size > 0:
         (chunk_id, chunk_size) = data.parse("4b 32u")
         data_size -= 8
-        if ((chunk_size % 2) == 1):
+        if (chunk_size % 2) == 1:
             chunk_size += 1
         yield chunk_id
-        if (chunk_id != 'data'):
+        if chunk_id != b"data":
             data.skip_bytes(chunk_size)
             data_size -= chunk_size
 
@@ -49,15 +47,15 @@ def __riff_chunk_ids__(data_size, data):
 class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
     """a WavPack audio file"""
 
-    from audiotools.text import (COMP_WAVPACK_VERYFAST,
+    from audiotools.text import (COMP_WAVPACK_FAST,
                                  COMP_WAVPACK_VERYHIGH)
 
     SUFFIX = "wv"
     NAME = SUFFIX
     DESCRIPTION = u"WavPack"
     DEFAULT_COMPRESSION = "standard"
-    COMPRESSION_MODES = ("veryfast", "fast", "standard", "high", "veryhigh")
-    COMPRESSION_DESCRIPTIONS = {"veryfast": COMP_WAVPACK_VERYFAST,
+    COMPRESSION_MODES = ("fast", "standard", "high", "veryhigh")
+    COMPRESSION_DESCRIPTIONS = {"fast": COMP_WAVPACK_FAST,
                                 "veryhigh": COMP_WAVPACK_VERYHIGH}
 
     BITS_PER_SAMPLE = (8, 16, 24, 32)
@@ -66,32 +64,6 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                      32000, 44100, 48000,  64000,
                      88200, 96000, 192000, 0)
 
-    __options__ = {"veryfast": {"block_size": 44100,
-                                "joint_stereo": True,
-                                "false_stereo": True,
-                                "wasted_bits": True,
-                                "correlation_passes": 1},
-                   "fast": {"block_size": 44100,
-                            "joint_stereo": True,
-                            "false_stereo": True,
-                            "wasted_bits": True,
-                            "correlation_passes": 2},
-                   "standard": {"block_size": 44100,
-                                "joint_stereo": True,
-                                "false_stereo": True,
-                                "wasted_bits": True,
-                                "correlation_passes": 5},
-                   "high": {"block_size": 44100,
-                            "joint_stereo": True,
-                            "false_stereo": True,
-                            "wasted_bits": True,
-                            "correlation_passes": 10},
-                   "veryhigh": {"block_size": 44100,
-                                "joint_stereo": True,
-                                "false_stereo": True,
-                                "wasted_bits": True,
-                                "correlation_passes": 16}}
-
     def __init__(self, filename):
         """filename is a plain string"""
 
@@ -128,7 +100,7 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         raises IOError if unable to read the file"""
 
         metadata = ApeTaggedAudio.get_metadata(self)
-        if (metadata is not None):
+        if metadata is not None:
             metadata.frame_count = self.total_frames()
         return metadata
 
@@ -141,11 +113,11 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         to avoid losing those chunks"""
 
         for (sub_header, nondecoder, data_size, data) in self.sub_blocks():
-            if ((sub_header == 1) and nondecoder):
+            if (sub_header == 1) and nondecoder:
                 if (set(__riff_chunk_ids__(data_size,
-                                           data)) != {'fmt ', 'data'}):
+                                           data)) != {b"fmt ", b"data"}):
                     return True
-            elif ((sub_header == 2) and nondecoder):
+            elif (sub_header == 2) and nondecoder:
                 return True
         else:
             return False
@@ -164,13 +136,13 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         tail = None
 
         for (sub_block_id, nondecoder, data_size, data) in self.sub_blocks():
-            if ((sub_block_id == 1) and nondecoder):
+            if (sub_block_id == 1) and nondecoder:
                 head = data.read_bytes(data_size)
-            elif ((sub_block_id == 2) and nondecoder):
+            elif (sub_block_id == 2) and nondecoder:
                 tail = data.read_bytes(data_size)
 
-        if (head is not None):
-            return (head, tail if tail is not None else "")
+        if head is not None:
+            return (head, tail if tail is not None else b"")
         else:
             raise ValueError("no wave header found")
 
@@ -214,16 +186,19 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
 
         try:
             (encode_wavpack if encoding_function is None
-             else encoding_function)(filename,
-                                     BufferedPCMReader(counter),
-                                     wave_header=header,
-                                     wave_footer=footer,
-                                     **cls.__options__[compression])
+             else encoding_function)(
+                filename=filename,
+                pcmreader=counter,
+                compression=compression,
+                wave_header=header,
+                wave_footer=footer)
+
+            counter.close()
 
             data_bytes_written = counter.bytes_written()
 
             # ensure output data size matches the "data" chunk's size
-            if (data_size != data_bytes_written):
+            if data_size != data_bytes_written:
                 from audiotools.text import ERR_WAV_TRUNCATED_DATA_CHUNK
                 raise EncodingError(ERR_WAV_TRUNCATED_DATA_CHUNK)
 
@@ -234,15 +209,17 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                 raise EncodingError(str(err))
 
             # ensure total size is correct
-            if ((len(header) + data_size + len(footer)) != total_size):
+            if (len(header) + data_size + len(footer)) != total_size:
                 from audiotools.text import ERR_WAV_INVALID_SIZE
                 raise EncodingError(ERR_WAV_INVALID_SIZE)
 
             return cls(filename)
         except (ValueError, IOError) as msg:
+            counter.close()
             cls.__unlink__(filename)
             raise EncodingError(str(msg))
         except Exception as err:
+            counter.close()
             cls.__unlink__(filename)
             raise err
 
@@ -255,9 +232,9 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
 
         def blocks_iter(reader):
             try:
-                while (True):
+                while True:
                     (wvpk, block_size) = reader.parse("4b 32u 192p")
-                    if (wvpk == 'wvpk'):
+                    if wvpk == b"wvpk":
                         yield (block_size - 24,
                                reader.substream(block_size - 24))
                     else:
@@ -265,15 +242,12 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
             except IOError:
                 return
 
-        if (reader is None):
+        if reader is None:
             from audiotools.bitstream import BitstreamReader
 
-            reader = BitstreamReader(open(self.filename), 1)
-            try:
+            with BitstreamReader(open(self.filename, "rb"), True) as reader:
                 for block in blocks_iter(reader):
                     yield block
-            finally:
-                reader.close()
         else:
             for block in blocks_iter(reader):
                 yield block
@@ -287,20 +261,20 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         """
 
         for (block_size, block_data) in self.blocks(reader):
-            while (block_size > 0):
+            while block_size > 0:
                 (metadata_function,
                  nondecoder_data,
                  actual_size_1_less,
                  large_block) = block_data.parse("5u 1u 1u 1u")
 
-                if (large_block):
+                if large_block:
                     sub_block_size = block_data.read(24)
                     block_size -= 4
                 else:
                     sub_block_size = block_data.read(8)
                     block_size -= 2
 
-                if (actual_size_1_less):
+                if actual_size_1_less:
                     yield (metadata_function,
                            nondecoder_data,
                            sub_block_size * 2 - 1,
@@ -318,9 +292,9 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         from audiotools.bitstream import BitstreamReader
         from audiotools import ChannelMask
 
-        reader = BitstreamReader(open(self.filename, "rb"), 1)
-        reader.mark()
-        try:
+        with BitstreamReader(open(self.filename, "rb"), True) as reader:
+            pos = reader.getpos()
+
             (block_id,
              total_samples,
              bits_per_sample,
@@ -330,11 +304,11 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
              sample_rate) = reader.parse(
                 "4b 64p 32u 64p 2u 1u 8p 1u 1u 5p 5p 4u 37p")
 
-            if (block_id != 'wvpk'):
+            if block_id != b"wvpk":
                 from audiotools.text import ERR_WAVPACK_INVALID_HEADER
                 raise InvalidWavPack(ERR_WAVPACK_INVALID_HEADER)
 
-            if (sample_rate != 0xF):
+            if sample_rate != 0xF:
                 self.__samplerate__ = WavPackAudio.SAMPLING_RATE[sample_rate]
             else:
                 # if unknown, pull from SAMPLE_RATE sub-block
@@ -342,21 +316,21 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                      nondecoder,
                      data_size,
                      data) in self.sub_blocks(reader):
-                    if ((block_id == 0x7) and nondecoder):
+                    if (block_id == 0x7) and nondecoder:
                         self.__samplerate__ = data.read(data_size * 8)
                         break
                 else:
                     # no SAMPLE RATE sub-block found
                     # so pull info from FMT chunk
-                    reader.rewind()
+                    reader.setpos(pos)
                     (self.__samplerate__,) = self.fmt_chunk(reader).parse(
                         "32p 32u")
 
             self.__bitspersample__ = [8, 16, 24, 32][bits_per_sample]
             self.__total_frames__ = total_samples
 
-            if (initial_block and final_block):
-                if (mono_output):
+            if initial_block and final_block:
+                if mono_output:
                     self.__channels__ = 1
                     self.__channel_mask__ = ChannelMask(0x4)
                 else:
@@ -364,12 +338,12 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                     self.__channel_mask__ = ChannelMask(0x3)
             else:
                 # if not mono or stereo, pull from CHANNEL INFO sub-block
-                reader.rewind()
+                reader.setpos(pos)
                 for (block_id,
                      nondecoder,
                      data_size,
                      data) in self.sub_blocks(reader):
-                    if ((block_id == 0xD) and not nondecoder):
+                    if (block_id == 0xD) and not nondecoder:
                         self.__channels__ = data.read(8)
                         self.__channel_mask__ = ChannelMask(
                             data.read((data_size - 1) * 8))
@@ -377,11 +351,11 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                 else:
                     # no CHANNEL INFO sub-block found
                     # so pull info from FMT chunk
-                    reader.rewind()
+                    reader.setpos(pos)
                     fmt = self.fmt_chunk(reader)
                     compression_code = fmt.read(16)
                     self.__channels__ = fmt.read(16)
-                    if (compression_code == 1):
+                    if compression_code == 1:
                         # this is theoretically possible
                         # with very old .wav files,
                         # but shouldn't happen in practice
@@ -408,7 +382,7 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                                                         front_center=True,
                                                         low_frequency=True)
                              }.get(self.__channels__, ChannelMask(0))
-                    elif (compression_code == 0xFFFE):
+                    elif compression_code == 0xFFFE:
                         fmt.skip(128)
                         mask = fmt.read(32)
                         self.__channel_mask__ = ChannelMask(mask)
@@ -416,10 +390,6 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
                         from audiotools.text import ERR_WAVPACK_UNSUPPORTED_FMT
                         raise InvalidWavPack(ERR_WAVPACK_UNSUPPORTED_FMT)
 
-        finally:
-            reader.unmark()
-            reader.close()
-
     def bits_per_sample(self):
         """returns an integer number of bits-per-sample this track contains"""
 
@@ -445,6 +415,17 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
 
         return True
 
+    @classmethod
+    def supports_from_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .from_pcm() classmethod"""
+
+        try:
+            from audiotools.encoders import encode_wavpack
+            return True
+        except ImportError:
+            return False
+
     @classmethod
     def from_pcm(cls, filename, pcmreader,
                  compression=None,
@@ -474,15 +455,18 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         try:
             (encode_wavpack if encoding_function is None
              else encoding_function)(
-                filename,
-                BufferedPCMReader(counter),
+                filename=filename,
+                pcmreader=counter,
                 total_pcm_frames=(total_pcm_frames if
                                   total_pcm_frames is not None else 0),
-                **cls.__options__[compression])
+                compression=compression)
+            counter.close()
         except (ValueError, IOError) as msg:
+            counter.close()
             cls.__unlink__(filename)
             raise EncodingError(str(msg))
         except Exception:
+            counter.close()
             cls.__unlink__(filename)
             raise
 
@@ -495,16 +479,27 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
 
         return cls(filename)
 
+    @classmethod
+    def supports_to_pcm(cls):
+        """returns True if all necessary components are available
+        to support the .from_pcm() classmethod"""
+
+        try:
+            from audiotools.decoders import WavPackDecoder
+            return True
+        except ImportError:
+            return False
+
     def to_pcm(self):
         """returns a PCMReader object containing the track's PCM data"""
 
-        from audiotools import decoders
+        from audiotools.decoders import WavPackDecoder
         from audiotools import PCMReaderError
 
         try:
-            return decoders.WavPackDecoder(open(self.filename, "rb"))
-        except (IOError, ValueError) as msg:
-            return PCMReaderError(error_message=str(msg),
+            return WavPackDecoder(self.filename)
+        except IOError as err:
+            return PCMReaderError(error_message=str(err),
                                   sample_rate=self.__samplerate__,
                                   channels=self.__channels__,
                                   channel_mask=int(self.channel_mask()),
@@ -517,17 +512,17 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
              nondecoder,
              data_size,
              data) in self.sub_blocks(reader):
-            if ((block_id == 1) and nondecoder):
+            if (block_id == 1) and nondecoder:
                 (riff, wave) = data.parse("4b 32p 4b")
-                if ((riff != 'RIFF') or (wave != 'WAVE')):
+                if (riff != b"RIFF") or (wave != b"WAVE"):
                     from audiotools.text import ERR_WAVPACK_INVALID_FMT
                     raise InvalidWavPack(ERR_WAVPACK_INVALID_FMT)
                 else:
-                    while (True):
+                    while True:
                         (chunk_id, chunk_size) = data.parse("4b 32u")
-                        if (chunk_id == 'fmt '):
+                        if chunk_id == b"fmt ":
                             return data.substream(chunk_size)
-                        elif (chunk_id == 'data'):
+                        elif chunk_id == b"data":
                             from audiotools.text import ERR_WAVPACK_INVALID_FMT
                             raise InvalidWavPack(ERR_WAVPACK_INVALID_FMT)
                         else:
@@ -546,14 +541,21 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         raises IOError if a problem occurs when reading the file"""
 
         from audiotools import cue as cue
+        from audiotools import SheetException
 
         metadata = self.get_metadata()
 
-        if ((metadata is not None) and ('Cuesheet' in metadata.keys())):
+        if (metadata is not None):
             try:
-                return cue.read_cuesheet_string(
-                    unicode(metadata['Cuesheet']).encode('utf-8', 'replace'))
-            except cue.CueException:
+                if (b'Cuesheet' in metadata.keys()):
+                    return cue.read_cuesheet_string(
+                        metadata[b'Cuesheet'].__unicode__())
+                elif (b'CUESHEET' in metadata.keys()):
+                    return cue.read_cuesheet_string(
+                        metadata[b'CUESHEET'].__unicode__())
+                else:
+                    return None;
+            except SheetException:
                 # unlike FLAC, just because a cuesheet is embedded
                 # does not mean it is compliant
                 return None
@@ -572,20 +574,20 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
         from audiotools.cue import write_cuesheet
         from audiotools.ape import ApeTag
 
-        if (cuesheet is None):
+        if cuesheet is None:
             return self.delete_cuesheet()
 
         metadata = self.get_metadata()
-        if (metadata is None):
+        if metadata is None:
             metadata = ApeTag([])
 
         cuesheet_data = BytesIO()
         write_cuesheet(cuesheet,
-                       str(Filename(self.filename).basename()),
+                       u"%s" % (Filename(self.filename).basename(),),
                        cuesheet_data)
 
-        metadata['Cuesheet'] = ApeTag.ITEM.string(
-            'Cuesheet',
+        metadata[b'Cuesheet'] = ApeTag.ITEM.string(
+            b'Cuesheet',
             cuesheet_data.getvalue().decode(FS_ENCODING, 'replace'))
 
         self.update_metadata(metadata)
@@ -595,7 +597,13 @@ class WavPackAudio(ApeTaggedAudio, ApeGainedAudio, WaveContainer):
 
         Raises IOError if a problem occurs when updating the file"""
 
+        from audiotools.ape import ApeTag
+
         metadata = self.get_metadata()
-        if ((metadata is not None) and ('Cuesheet' in metadata)):
-            del(metadata['Cuesheet'])
-            self.update_metadata(metadata)
+        if ((metadata is not None) and isinstance(metadata, ApeTag)):
+            if b"Cuesheet" in metadata.keys():
+                del(metadata[b"Cuesheet"])
+                self.update_metadata(metadata)
+            elif b"CUESHEET" in metadata.keys():
+                del(metadata[b"CUESHEET"])
+                self.update_metadata(metadata)
