@@ -1,9 +1,9 @@
 #include "sine.h"
-#include "../pcmconv.h"
+#include "../framelist.h"
 
 /********************************************************
  Audio Tools, a module and set of tools for manipulating audio data
- Copyright (C) 2007-2014  Brian Langenberger
+ Copyright (C) 2007-2015  Brian Langenberger
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@ Sine_Mono_init(decoders_Sine_Mono* self, PyObject *args, PyObject *kwds) {
     double f1;
     double f2;
 
-    self->buffer = aa_int_new();
     if ((self->audiotools_pcm = open_audiotools_pcm()) == NULL)
         return -1;
 
@@ -80,7 +79,6 @@ Sine_Mono_init(decoders_Sine_Mono* self, PyObject *args, PyObject *kwds) {
 }
 
 void Sine_Mono_dealloc(decoders_Sine_Mono* self) {
-    self->buffer->del(self->buffer);
     Py_XDECREF(self->audiotools_pcm);
 
     Py_TYPE(self)->tp_free((PyObject*)self);
@@ -98,12 +96,12 @@ Sine_Mono_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 
 static PyObject*
 Sine_Mono_read(decoders_Sine_Mono* self, PyObject* args) {
+    pcm_FrameList *framelist;
     int requested_frames;
     int frames_to_read;
-    int i;
     double d;
     int ia;
-    a_int* buffer1;
+    int *samples;
 
     if (self->closed) {
         PyErr_SetString(PyExc_ValueError, "cannot read closed stream");
@@ -114,25 +112,28 @@ Sine_Mono_read(decoders_Sine_Mono* self, PyObject* args) {
         return NULL;
 
     frames_to_read = MIN(MAX(requested_frames, 1), self->remaining_pcm_frames);
+    self->remaining_pcm_frames -= frames_to_read;
 
-    self->buffer->reset(self->buffer);
-    buffer1 = self->buffer->append(self->buffer);
+    framelist = new_FrameList(self->audiotools_pcm,
+                              1,
+                              self->bits_per_sample,
+                              frames_to_read);
 
-    for (i = 0; i < frames_to_read; i++) {
+    samples = framelist->samples;
+
+    for (; frames_to_read; frames_to_read--) {
         d = ((self->a1 * sin(self->theta1)) +
              (self->a2 * sin(self->theta2))) * (double)(self->full_scale);
 
         ia = (int)(d + 0.5);
-        buffer1->append(buffer1, ia);
+        *samples = ia;
+        samples++;
         self->theta1 += self->delta1;
         self->theta2 += self->delta2;
     }
 
-    self->remaining_pcm_frames -= frames_to_read;
 
-    return aa_int_to_FrameList(self->audiotools_pcm,
-                               self->buffer,
-                               self->bits_per_sample);
+    return (PyObject*)framelist;
 }
 
 static PyObject*
@@ -153,6 +154,22 @@ Sine_Mono_reset(decoders_Sine_Mono* self, PyObject* args) {
     return Py_None;
 }
 
+static PyObject*
+Sine_Mono_enter(decoders_Sine_Mono* self, PyObject *args)
+{
+    Py_INCREF(self);
+    return (PyObject *)self;
+}
+
+static PyObject*
+Sine_Mono_exit(decoders_Sine_Mono* self, PyObject *args)
+{
+    self->closed = 1;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject*
 Sine_Mono_channels(decoders_Sine_Mono *self, void *closure) {
     return Py_BuildValue("i", 1);
@@ -179,8 +196,6 @@ Sine_Stereo_init(decoders_Sine_Stereo* self, PyObject *args, PyObject *kwds) {
     double f1;
     double f2;
 
-    self->buffer = aa_int_new();
-
     if ((self->audiotools_pcm = open_audiotools_pcm()) == NULL)
         return -1;
 
@@ -229,7 +244,6 @@ Sine_Stereo_init(decoders_Sine_Stereo* self, PyObject *args, PyObject *kwds) {
 }
 
 void Sine_Stereo_dealloc(decoders_Sine_Stereo* self) {
-    self->buffer->del(self->buffer);
     Py_XDECREF(self->audiotools_pcm);
 
     Py_TYPE(self)->tp_free((PyObject*)self);
@@ -246,13 +260,13 @@ Sine_Stereo_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 
 static PyObject*
 Sine_Stereo_read(decoders_Sine_Stereo* self, PyObject* args) {
+    pcm_FrameList *framelist;
     int requested_frames;
     int frames_to_read;
     int i;
     double d;
     int ia;
-    a_int* buffer1;
-    a_int* buffer2;
+    int *samples;
 
     if (self->closed) {
         PyErr_SetString(PyExc_ValueError, "cannot read closed stream");
@@ -264,29 +278,29 @@ Sine_Stereo_read(decoders_Sine_Stereo* self, PyObject* args) {
 
     frames_to_read = MIN(MAX(requested_frames, 1), self->remaining_pcm_frames);
 
-    self->buffer->reset(self->buffer);
-    buffer1 = self->buffer->append(self->buffer);
-    buffer2 = self->buffer->append(self->buffer);
+    framelist = new_FrameList(self->audiotools_pcm,
+                              2,
+                              self->bits_per_sample,
+                              frames_to_read);
+    samples = framelist->samples;
 
     for (i = 0; i < frames_to_read; i++) {
         d = ((self->a1 * sin(self->theta1)) +
              (self->a2 * sin(self->theta2))) * (double)(self->full_scale);
         ia = (int)(d + 0.5);
-        buffer1->append(buffer1, ia);
+        put_sample(samples, 0, 2, i, ia);
         d = -((self->a1 * sin(self->theta1 * self->fmult)) +
               (self->a2 * sin(self->theta2 * self->fmult))) *
             (double)(self->full_scale);
         ia = (int)(d + 0.5);
-        buffer2->append(buffer2, ia);
+        put_sample(samples, 1, 2, i, ia);
         self->theta1 += self->delta1;
         self->theta2 += self->delta2;
     }
 
     self->remaining_pcm_frames -= frames_to_read;
 
-    return aa_int_to_FrameList(self->audiotools_pcm,
-                               self->buffer,
-                               self->bits_per_sample);
+    return (PyObject*)framelist;
 }
 
 static PyObject*
@@ -307,6 +321,22 @@ Sine_Stereo_reset(decoders_Sine_Stereo* self, PyObject* args) {
     return Py_None;
 }
 
+static PyObject*
+Sine_Stereo_enter(decoders_Sine_Stereo* self, PyObject *args)
+{
+    Py_INCREF(self);
+    return (PyObject *)self;
+}
+
+static PyObject*
+Sine_Stereo_exit(decoders_Sine_Stereo* self, PyObject *args)
+{
+    self->closed = 1;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject*
 Sine_Stereo_channels(decoders_Sine_Stereo *self, void *closure) {
     return Py_BuildValue("i", 2);
@@ -330,8 +360,6 @@ Sine_Stereo_channel_mask(decoders_Sine_Stereo *self, void *closure) {
 
 int
 Sine_Simple_init(decoders_Sine_Simple* self, PyObject *args, PyObject *kwds) {
-    self->buffer = aa_int_new();
-
     if ((self->audiotools_pcm = open_audiotools_pcm()) == NULL)
         return -1;
 
@@ -373,7 +401,6 @@ Sine_Simple_init(decoders_Sine_Simple* self, PyObject *args, PyObject *kwds) {
 }
 
 void Sine_Simple_dealloc(decoders_Sine_Simple* self) {
-    self->buffer->del(self->buffer);
     Py_XDECREF(self->audiotools_pcm);
 
     Py_TYPE(self)->tp_free((PyObject*)self);
@@ -391,41 +418,45 @@ Sine_Simple_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 
 static PyObject*
 Sine_Simple_read(decoders_Sine_Simple* self, PyObject* args) {
+    pcm_FrameList *framelist;
     int requested_frames;
     int frames_to_read;
     int i;
-    a_int* buffer;
     double d;
     int ia;
+    int *samples;
 
     if (self->closed) {
         PyErr_SetString(PyExc_ValueError, "cannot read closed stream");
         return NULL;
     }
 
-
     if (!PyArg_ParseTuple(args, "i", &requested_frames))
         return NULL;
 
-    self->buffer->reset(self->buffer);
-    buffer = self->buffer->append(self->buffer);
-
     frames_to_read = MIN(MAX(requested_frames, 1), self->remaining_pcm_frames);
 
+    framelist = new_FrameList(self->audiotools_pcm,
+                              1,
+                              self->bits_per_sample,
+                              frames_to_read);
+
+    samples = framelist->samples;
+
     for (i = 0; i < frames_to_read; i++) {
         d = (double)(self->max_value) *
             sin(((M_PI * 2) *
                  (double)(self->i % self->count)) /
                 (double)(self->count));
         ia = (int)(round(d));
-        buffer->append(buffer, ia);
+        *samples = ia;
+        samples++;
         self->i += 1;
     }
 
     self->remaining_pcm_frames -= frames_to_read;
-    return aa_int_to_FrameList(self->audiotools_pcm,
-                               self->buffer,
-                               self->bits_per_sample);
+
+    return (PyObject*)framelist;
 }
 
 static PyObject*
@@ -446,6 +477,22 @@ Sine_Simple_reset(decoders_Sine_Simple* self, PyObject* args) {
     return Py_None;
 }
 
+static PyObject*
+Sine_Simple_enter(decoders_Sine_Simple* self, PyObject *args)
+{
+    Py_INCREF(self);
+    return (PyObject *)self;
+}
+
+static PyObject*
+Sine_Simple_exit(decoders_Sine_Simple* self, PyObject *args)
+{
+    self->closed = 1;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject*
 Sine_Simple_channels(decoders_Sine_Simple *self, void *closure) {
     return Py_BuildValue("i", 1);
@@ -488,7 +535,6 @@ SameSample_init(decoders_SameSample* self, PyObject *args, PyObject *kwds)
                              NULL};
 
     self->closed = 0;
-    self->buffer = a_int_new();
     if ((self->audiotools_pcm = open_audiotools_pcm()) == NULL)
         return -1;
 
@@ -547,7 +593,6 @@ SameSample_init(decoders_SameSample* self, PyObject *args, PyObject *kwds)
 
 void SameSample_dealloc(decoders_SameSample* self)
 {
-    self->buffer->del(self->buffer);
     Py_XDECREF(self->audiotools_pcm);
 
     Py_TYPE(self)->tp_free((PyObject*)self);
@@ -557,6 +602,10 @@ static PyObject*
 SameSample_read(decoders_SameSample* self, PyObject* args)
 {
     int pcm_frames;
+    unsigned total_samples;
+    pcm_FrameList *framelist;
+    const int sample = self->sample;
+    int *samples;
 
     if (self->closed) {
         PyErr_SetString(PyExc_ValueError, "unable to read closed stream");
@@ -568,16 +617,23 @@ SameSample_read(decoders_SameSample* self, PyObject* args)
 
     pcm_frames = MIN(MAX(pcm_frames, 1), self->remaining_pcm_frames);
 
-    self->buffer->mset(self->buffer,
-                       pcm_frames * self->channels,
-                       self->sample);
+    framelist = new_FrameList(self->audiotools_pcm,
+                              self->channels,
+                              self->bits_per_sample,
+                              pcm_frames);
+
+    samples = framelist->samples;
+
+    for (total_samples = pcm_frames * self->channels;
+         total_samples;
+         total_samples--) {
+         *samples = sample;
+         samples++;
+    }
 
     self->remaining_pcm_frames -= pcm_frames;
 
-    return a_int_to_FrameList(self->audiotools_pcm,
-                              self->buffer,
-                              self->channels,
-                              self->bits_per_sample);
+    return (PyObject*)framelist;
 }
 
 static PyObject*
@@ -599,6 +655,22 @@ SameSample_reset(decoders_SameSample* self, PyObject* args)
     return Py_None;
 }
 
+static PyObject*
+SameSample_enter(decoders_SameSample* self, PyObject *args)
+{
+    Py_INCREF(self);
+    return (PyObject *)self;
+}
+
+static PyObject*
+SameSample_exit(decoders_SameSample* self, PyObject *args)
+{
+    self->closed = 1;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 static PyObject*
 SameSample_channels(decoders_SameSample *self, void *closure)
 {
