@@ -5,7 +5,7 @@
 
 /********************************************************
  Audio Tools, a module and set of tools for manipulating audio data
- Copyright (C) 2007-2014  Brian Langenberger
+ Copyright (C) 2007-2015  Brian Langenberger
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -30,13 +30,5 @@ MOD_INIT(encoders)
 
     MOD_DEF(m, "encoders", "low-level audio format encoders",  module_methods)
 
-    encoders_ALACEncoderType.tp_new = PyType_GenericNew;
-    if (PyType_Ready(&encoders_ALACEncoderType) < 0)
-        return MOD_ERROR_VAL;
-
-    Py_INCREF(&encoders_ALACEncoderType);
-    PyModule_AddObject(m, "ALACEncoder",
-                       (PyObject *)&encoders_ALACEncoderType);
-
     return MOD_SUCCESS_VAL(m);
 }
