@@ -1,27 +1,37 @@
+/********************************************************
+ Bitstream Library, a module for reading bits of data
+
+ Copyright (C) 2007-2015  Brian Langenberger
+
+ The Bitstream Library is free software; you can redistribute it and/or modify
+ it under the terms of either:
+
+   * the GNU Lesser General Public License as published by the Free
+     Software Foundation; either version 3 of the License, or (at your
+     option) any later version.
+
+ or
+
+   * the GNU General Public License as published by the Free Software
+     Foundation; either version 2 of the License, or (at your option) any
+     later version.
+
+ or both in parallel, as here.
+
+ The Bitstream Library is distributed in the hope that it will be useful, but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ for more details.
+
+ You should have received copies of the GNU General Public License and the
+ GNU Lesser General Public License along with the GNU MP Library.  If not,
+ see https://www.gnu.org/licenses/.
+ *******************************************************/
+
 #include "huffman.h"
 #include <stdlib.h>
 #include <string.h>
 
-
-/********************************************************
- Audio Tools, a module and set of tools for manipulating audio data
- Copyright (C) 2007-2014  Brian Langenberger
-
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
-
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
-
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-*******************************************************/
-
 typedef enum {NODE_TREE, NODE_LEAF} huffman_node_type;
 
 struct huffman_node {
@@ -557,6 +567,24 @@ bw_fix_tree(bw_huffman_table_t *table,
     }
 }
 
+struct huffman_frequency
+bw_str_to_frequency(const char *s, int value)
+{
+    struct huffman_frequency frequency;
+
+    frequency.bits = 0;
+    frequency.length = 0;
+
+    for (; *s != '\0'; s++) {
+        frequency.bits = ((frequency.bits << 1) | (*s == '0' ? 0 : 1));
+        frequency.length++;
+    }
+
+    frequency.value = value;
+
+    return frequency;
+}
+
 #ifdef EXECUTABLE
 
 #include "parson.h"
