@@ -6,7 +6,7 @@ typedef int Py_ssize_t;
 
 /********************************************************
  Audio Tools, a module and set of tools for manipulating audio data
- Copyright (C) 2007-2014  Brian Langenberger
+ Copyright (C) 2007-2015  Brian Langenberger
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -27,9 +27,6 @@ typedef int Py_ssize_t;
 #define IS_PY3K
 #endif
 
-static PyObject*
-BitstreamReader_Substream(PyObject *dummy, PyObject *args);
-
 PyObject*
 bitstream_format_size(PyObject *dummy, PyObject *args);
 
@@ -43,8 +40,6 @@ PyObject*
 bitstream_build_func(PyObject *dummy, PyObject *args);
 
 PyMethodDef module_methods[] = {
-    {"Substream", (PyCFunction)BitstreamReader_Substream,
-     METH_VARARGS, "build a fresh Substream BitstreamReader"}, /*FIXME*/
     {"format_size", (PyCFunction)bitstream_format_size,
      METH_VARARGS, "Calculate size of format string in bits"}, /*FIXME*/
     {"format_byte_size", (PyCFunction)bitstream_format_byte_size,
@@ -56,34 +51,20 @@ PyMethodDef module_methods[] = {
     {NULL}
 };
 
-typedef PyObject* (*read_object_f)(BitstreamReader *br,
-                                   unsigned bits);
-
 /*the BitstreamReader object
   a simple wrapper around our Bitstream reading struct*/
 
 typedef struct {
     PyObject_HEAD
 
-    PyObject* file_obj;
     BitstreamReader* bitstream;
-    int little_endian;
-    struct bs_buffer *string_buffer;
-    read_object_f read_unsigned;
-    read_object_f read_signed;
 } bitstream_BitstreamReader;
 
 static PyObject*
-brpy_read_unsigned_be(BitstreamReader *br, unsigned bits);
+brpy_read_unsigned(BitstreamReader *br, unsigned bits);
 
 static PyObject*
-brpy_read_unsigned_le(BitstreamReader *br, unsigned bits);
-
-static PyObject*
-brpy_read_signed_be(BitstreamReader *br, unsigned bits);
-
-static PyObject*
-brpy_read_signed_le(BitstreamReader *br, unsigned bits);
+brpy_read_signed(BitstreamReader *br, unsigned bits);
 
 /*reads byte_count bytes from reader to buffer
   returns 0 on success, 1 if a read error occurs with PyErr set accordingly*/
@@ -163,16 +144,13 @@ static PyObject*
 BitstreamReader_close(bitstream_BitstreamReader *self, PyObject *args);
 
 static PyObject*
-BitstreamReader_mark(bitstream_BitstreamReader *self, PyObject *args);
+BitstreamReader_getpos(bitstream_BitstreamReader *self, PyObject *args);
 
 static PyObject*
-BitstreamReader_has_mark(bitstream_BitstreamReader *self, PyObject *args);
+BitstreamReader_setpos(bitstream_BitstreamReader *self, PyObject *args);
 
 static PyObject*
-BitstreamReader_rewind(bitstream_BitstreamReader *self, PyObject *args);
-
-static PyObject*
-BitstreamReader_unmark(bitstream_BitstreamReader *self, PyObject *args);
+BitstreamReader_seek(bitstream_BitstreamReader *self, PyObject *args);
 
 static PyObject*
 BitstreamReader_add_callback(bitstream_BitstreamReader *self, PyObject *args);
@@ -184,11 +162,7 @@ static PyObject*
 BitstreamReader_call_callbacks(bitstream_BitstreamReader *self, PyObject *args);
 
 static PyObject*
-BitstreamReader_substream_meth(bitstream_BitstreamReader *self, PyObject *args);
-
-static PyObject*
-BitstreamReader_substream_append(bitstream_BitstreamReader *self,
-                                 PyObject *args);
+BitstreamReader_substream(bitstream_BitstreamReader *self, PyObject *args);
 
 static PyObject*
 BitstreamReader_parse(bitstream_BitstreamReader *self, PyObject *args);
@@ -196,6 +170,12 @@ BitstreamReader_parse(bitstream_BitstreamReader *self, PyObject *args);
 int
 BitstreamReader_init(bitstream_BitstreamReader *self, PyObject *args);
 
+static PyObject*
+BitstreamReader_enter(bitstream_BitstreamReader *self, PyObject *args);
+
+static PyObject*
+BitstreamReader_exit(bitstream_BitstreamReader *self, PyObject *args);
+
 PyMethodDef BitstreamReader_methods[] = {
     {"read", (PyCFunction)BitstreamReader_read, METH_VARARGS,
      "read(bits) -> unsigned int"},
@@ -246,19 +226,16 @@ PyMethodDef BitstreamReader_methods[] = {
     {"close", (PyCFunction)BitstreamReader_close, METH_NOARGS,
      "close()\n"
      "closes the stream and any underlying file object"},
-    {"mark", (PyCFunction)BitstreamReader_mark, METH_VARARGS,
-     "mark([mark_id])\n"
-     "pushes the current position onto a stack\n"
-     "which may be returned to with calls to rewind()\n"
-     "all marked positions should be unmarked when no longer needed"},
-    {"has_mark", (PyCFunction)BitstreamReader_has_mark, METH_VARARGS,
-     "has_mark([mark_id]) -> True if the given mark_id is present"},
-    {"rewind", (PyCFunction)BitstreamReader_rewind, METH_VARARGS,
-     "rewind([mark_id])\n"
-     "returns to the most recently marked position in the stream"},
-    {"unmark", (PyCFunction)BitstreamReader_unmark, METH_VARARGS,
-     "unmark([mark_id])\n"
-     "removes the most recently marked position from the stream"},
+    {"getpos", (PyCFunction)BitstreamReader_getpos, METH_NOARGS,
+     "getpos() -> position"},
+    {"setpos", (PyCFunction)BitstreamReader_setpos, METH_VARARGS,
+     "setpos(position)"},
+    {"seek", (PyCFunction)BitstreamReader_seek, METH_VARARGS,
+     "seek(position, whence)\n"
+     "positions the stream at the given position where\n"
+     "position is stream position in bytes and\n"
+     "whence 0 = stream start, 1 = current position, 2 = stream end\n"
+     "no callbacks are performed on intervening bytes"},
     {"add_callback", (PyCFunction)BitstreamReader_add_callback, METH_VARARGS,
      "add_callback(function)\n"
      "where \"function\" takes a single byte as an argument\n"
@@ -270,13 +247,13 @@ PyMethodDef BitstreamReader_methods[] = {
      METH_VARARGS,
      "call_callbacks(byte)\n"
      "calls the attached callbacks as if the byte had been read"},
-    {"substream", (PyCFunction)BitstreamReader_substream_meth, METH_VARARGS,
+    {"substream", (PyCFunction)BitstreamReader_substream, METH_VARARGS,
      "substream(bytes) -> BitstreamReader\n"
      "returns a sub-reader containing the given number of input bytes"},
-    {"substream_append", (PyCFunction)BitstreamReader_substream_append,
-     METH_VARARGS,
-     "substream_append(BitstreamReader, bytes)\n"
-     "appends an additional number of bytes to the given substream"},
+    {"__enter__", (PyCFunction)BitstreamReader_enter,
+     METH_NOARGS, "enter() -> self"},
+    {"__exit__", (PyCFunction)BitstreamReader_exit,
+     METH_VARARGS, "exit(exc_type, exc_value, traceback) -> None"},
     {NULL}
 };
 
@@ -339,14 +316,15 @@ typedef struct {
     bw_huffman_table_t* bw_table;
 } bitstream_HuffmanTree;
 
+static PyObject*
+HuffmanTree_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
+
 int
 HuffmanTree_init(bitstream_HuffmanTree *self, PyObject *args);
 
 void
 HuffmanTree_dealloc(bitstream_HuffmanTree *self);
 
-static PyObject*
-HuffmanTree_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
 PyTypeObject bitstream_HuffmanTreeType = {
     PyVarObject_HEAD_INIT(NULL, 0)
@@ -382,7 +360,6 @@ PyTypeObject bitstream_HuffmanTreeType = {
     0,                         /* tp_base */
     0,                         /* tp_dict */
     0,                         /* tp_descr_get */
-
     0,                         /* tp_descr_set */
     0,                         /* tp_dictoffset */
     (initproc)HuffmanTree_init, /* tp_init */
@@ -390,6 +367,67 @@ PyTypeObject bitstream_HuffmanTreeType = {
     HuffmanTree_new,          /* tp_new */
 };
 
+/*BitstreamReaderPosition is an opaque container for positions
+  returned by BitstreamReader.getpos()
+  it has no methods or attributes and can't even be instantiated directly*/
+typedef struct {
+    PyObject_HEAD
+
+    br_pos_t *pos;
+} bitstream_BitstreamReaderPosition;
+
+static PyObject*
+BitstreamReaderPosition_new(PyTypeObject *type, PyObject *args,
+                            PyObject *kwds);
+
+int
+BitstreamReaderPosition_init(bitstream_BitstreamReaderPosition *self,
+                             PyObject *args);
+
+void
+BitstreamReaderPosition_dealloc(bitstream_BitstreamReaderPosition *self);
+
+PyTypeObject bitstream_BitstreamReaderPositionType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "bitstream.BitstreamReaderPosition",    /*tp_name*/
+    sizeof(bitstream_BitstreamReaderPosition), /*tp_basicsize*/
+    0,                         /*tp_itemsize*/
+    (destructor)BitstreamReaderPosition_dealloc, /*tp_dealloc*/
+    0,                         /*tp_print*/
+    0,                         /*tp_getattr*/
+    0,                         /*tp_setattr*/
+    0,                         /*tp_compare*/
+    0,                         /*tp_repr*/
+    0,                         /*tp_as_number*/
+    0,                         /*tp_as_sequence*/
+    0,                         /*tp_as_mapping*/
+    0,                         /*tp_hash */
+    0,                         /*tp_call*/
+    0,                         /*tp_str*/
+    0,                         /*tp_getattro*/
+    0,                         /*tp_setattro*/
+    0,                         /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
+    "BitstreamReaderPosition", /* tp_doc */
+    0,                         /* tp_traverse */
+    0,                         /* tp_clear */
+    0,                         /* tp_richcompare */
+    0,                         /* tp_weaklistoffset */
+    0,                         /* tp_iter */
+    0,                         /* tp_iternext */
+    0,                         /* tp_methods */
+    0,                         /* tp_members */
+    0,                         /* tp_getset */
+    0,                         /* tp_base */
+    0,                         /* tp_dict */
+    0,                         /* tp_descr_get */
+    0,                         /* tp_descr_set */
+    0,                         /* tp_dictoffset */
+    (initproc)BitstreamReaderPosition_init, /* tp_init */
+    0,                         /* tp_alloc */
+    BitstreamReaderPosition_new, /* tp_new */
+};
+
 PyObject*
 bitstream_format_size(PyObject *dummy, PyObject *args)
 {
@@ -412,17 +450,10 @@ bitstream_format_byte_size(PyObject *dummy, PyObject *args)
     return Py_BuildValue("I", bs_format_byte_size(format_string));
 }
 
-typedef int (*write_object_f)(BitstreamWriter *bw,
-                              unsigned bits,
-                              PyObject *value);
-
 typedef struct {
     PyObject_HEAD
 
-    PyObject* file_obj;
     BitstreamWriter* bitstream;
-    write_object_f write_unsigned;
-    write_object_f write_signed;
 } bitstream_BitstreamWriter;
 
 static PyObject*
@@ -447,16 +478,10 @@ static int
 bw_validate_signed_range(unsigned bits, PyObject *value);
 
 static int
-bwpy_write_unsigned_be(BitstreamWriter *bw, unsigned bits, PyObject *value);
+bwpy_write_unsigned(BitstreamWriter *bw, unsigned bits, PyObject *value);
 
 static int
-bwpy_write_unsigned_le(BitstreamWriter *bw, unsigned bits, PyObject *value);
-
-static int
-bwpy_write_signed_be(BitstreamWriter *bw, unsigned bits, PyObject *value);
-
-static int
-bwpy_write_signed_le(BitstreamWriter *bw, unsigned bits, PyObject *value);
+bwpy_write_signed(BitstreamWriter *bw, unsigned bits, PyObject *value);
 
 static PyObject*
 BitstreamWriter_write(bitstream_BitstreamWriter *self, PyObject *args);
@@ -498,16 +523,12 @@ BitstreamWriter_call_callbacks(bitstream_BitstreamWriter *self,
                                PyObject *args);
 
 static PyObject*
-BitstreamWriter_mark(bitstream_BitstreamWriter *self, PyObject *args);
+BitstreamWriter_getpos(bitstream_BitstreamWriter *self,
+                       PyObject *args);
 
 static PyObject*
-BitstreamWriter_has_mark(bitstream_BitstreamWriter *self, PyObject *args);
-
-static PyObject*
-BitstreamWriter_rewind(bitstream_BitstreamWriter *self, PyObject *args);
-
-static PyObject*
-BitstreamWriter_unmark(bitstream_BitstreamWriter *self, PyObject *args);
+BitstreamWriter_setpos(bitstream_BitstreamWriter *self,
+                       PyObject *args);
 
 static PyObject*
 BitstreamWriter_build(bitstream_BitstreamWriter *self, PyObject *args);
@@ -518,6 +539,12 @@ BitstreamWriter_flush(bitstream_BitstreamWriter *self, PyObject *args);
 static PyObject*
 BitstreamWriter_close(bitstream_BitstreamWriter *self, PyObject *args);
 
+static PyObject*
+BitstreamWriter_enter(bitstream_BitstreamWriter *self, PyObject *args);
+
+static PyObject*
+BitstreamWriter_exit(bitstream_BitstreamWriter *self, PyObject *args);
+
 int
 BitstreamWriter_init(bitstream_BitstreamWriter *self, PyObject *args);
 
@@ -576,19 +603,14 @@ PyMethodDef BitstreamWriter_methods[] = {
      METH_VARARGS,
      "call_callbacks(byte)\n"
      "calls the attached callbacks as if the byte had been written"},
-    {"mark", (PyCFunction)BitstreamWriter_mark, METH_VARARGS,
-     "mark([mark_id])\n"
-     "pushes the current position onto a stack\n"
-     "which may be returned to with calls to rewind()\n"
-     "all marked positions should be unmarked when no longer needed"},
-    {"has_mark", (PyCFunction)BitstreamWriter_has_mark, METH_VARARGS,
-     "has_mark([mark_id]) -> True if the given mark_id is present"},
-    {"rewind", (PyCFunction)BitstreamWriter_rewind, METH_VARARGS,
-     "rewind([mark_id])\n"
-     "returns to the most recently marked position in the stream"},
-    {"unmark", (PyCFunction)BitstreamWriter_unmark, METH_VARARGS,
-     "unmark([mark_id])\n"
-     "removes the most recently marked position from the stream"},
+    {"getpos", (PyCFunction)BitstreamWriter_getpos, METH_NOARGS,
+     "getpos() -> position"},
+    {"setpos", (PyCFunction)BitstreamWriter_setpos, METH_VARARGS,
+     "setpos(position)"},
+    {"__enter__", (PyCFunction)BitstreamWriter_enter,
+     METH_NOARGS, "enter() -> self"},
+    {"__exit__", (PyCFunction)BitstreamWriter_exit,
+     METH_VARARGS, "exit(exc_type, exc_value, traceback) -> None"},
     {NULL}
 };
 
@@ -644,9 +666,7 @@ PyTypeObject bitstream_BitstreamWriterType = {
 typedef struct {
     PyObject_HEAD
 
-    BitstreamWriter* bitstream;
-    write_object_f write_unsigned;
-    write_object_f write_signed;
+    BitstreamRecorder* bitstream;
 } bitstream_BitstreamRecorder;
 
 static PyObject*
@@ -722,7 +742,7 @@ BitstreamRecorder_reset(bitstream_BitstreamRecorder *self,
                         PyObject *args);
 
 /*returns the internal BitstreamWriter struct of the given object
-  or NULL if it is not a BitstreamWriter/Recorder/Accumulator*/
+  or NULL if it is not a BitstreamWriter/Recorder*/
 static BitstreamWriter*
 internal_writer(PyObject *writer);
 
@@ -731,13 +751,23 @@ BitstreamRecorder_copy(bitstream_BitstreamRecorder *self,
                        PyObject *args);
 
 static PyObject*
-BitstreamRecorder_split(bitstream_BitstreamRecorder *self,
-                        PyObject *args);
+BitstreamRecorder_getpos(bitstream_BitstreamRecorder *self,
+                         PyObject *args);
+
+static PyObject*
+BitstreamRecorder_setpos(bitstream_BitstreamRecorder *self,
+                         PyObject *args);
 
 static PyObject*
 BitstreamRecorder_close(bitstream_BitstreamRecorder *self,
                         PyObject *args);
 
+static PyObject*
+BitstreamRecorder_enter(bitstream_BitstreamRecorder *self, PyObject *args);
+
+static PyObject*
+BitstreamRecorder_exit(bitstream_BitstreamRecorder *self, PyObject *args);
+
 int
 BitstreamRecorder_init(bitstream_BitstreamRecorder *self,
                        PyObject *args);
@@ -789,13 +819,7 @@ PyMethodDef BitstreamRecorder_methods[] = {
     {"copy", (PyCFunction)BitstreamRecorder_copy, METH_VARARGS,
      "copy(target)\n"
      "copies the written data to \"target\", which must be a\n"
-     "BitstreamWriter, Recorder or Accumulator"},
-    {"split", (PyCFunction)BitstreamRecorder_split, METH_VARARGS,
-     "split(target, remainder, bytes)\n"
-     "copies the given number of written bytes to \"target\"\n"
-     "and the remaining bytes to \"remainder\"\n"
-     "where \"target\" and \"remainder\" must be a\n"
-     "BitstreamWriter, Recorder, Accumulator or None"},
+     "BitstreamWriter or Recorder"},
     {"build", (PyCFunction)BitstreamRecorder_build, METH_VARARGS,
      "build(format_string, [value1, value2, ...])\n"
      "where \"format_string\" maps to the calls:\n"
@@ -811,17 +835,27 @@ PyMethodDef BitstreamRecorder_methods[] = {
     {"swap", (PyCFunction)BitstreamRecorder_swap, METH_VARARGS,
      "swap(recorder)\n"
      "swaps our written data with that of another BitstreamRecorder"},
-    {"add_callback", (PyCFunction)BitstreamRecorder_add_callback, METH_VARARGS,
+    {"add_callback", (PyCFunction)BitstreamRecorder_add_callback,
+     METH_VARARGS,
      "add_callback(function)\n"
      "where \"function\" takes a single byte as an argument\n"
      "and is called upon each written byte to the stream"},
-    {"pop_callback", (PyCFunction)BitstreamRecorder_pop_callback, METH_NOARGS,
+    {"pop_callback", (PyCFunction)BitstreamRecorder_pop_callback,
+     METH_NOARGS,
      "pop_callback() -> function\n"
      "removes and returns the most recently added callback"},
     {"call_callbacks", (PyCFunction)BitstreamRecorder_call_callbacks,
      METH_VARARGS,
      "call_callbacks(byte)\n"
      "calls the attached callbacks as if the byte had been written"},
+    {"getpos", (PyCFunction)BitstreamRecorder_getpos, METH_NOARGS,
+     "getpos() -> position"},
+    {"setpos", (PyCFunction)BitstreamRecorder_setpos, METH_VARARGS,
+     "setpos(position)"},
+    {"__enter__", (PyCFunction)BitstreamRecorder_enter,
+     METH_NOARGS, "enter() -> self"},
+    {"__exit__", (PyCFunction)BitstreamRecorder_exit,
+     METH_VARARGS, "exit(exc_type, exc_value, traceback) -> None"},
     {NULL}
 };
 
@@ -873,163 +907,32 @@ PyTypeObject bitstream_BitstreamRecorderType = {
     BitstreamRecorder_new,       /* tp_new */
 };
 
-/*given a BitstreamReader, format string and list object
-  parses the format from the reader and appends Python values to the list
-  returns 0 on success, 1 on failure (with PyErr set)*/
-int
-bitstream_parse(BitstreamReader* stream,
-                read_object_f read_unsigned,
-                read_object_f read_signed,
-                const char* format,
-                PyObject* values);
-
-/*given a BitstreamWriter, format string and PySequence of Python values,
-  writes those values to the writer
-  returns 0 on success, 1 on failure (with PyErr set)*/
-int
-bitstream_build(BitstreamWriter* stream,
-                write_object_f write_unsigned,
-                write_object_f write_signed,
-                const char* format,
-                PyObject* iterator);
-
+/*BitstreamWriterPosition is an opaque container for positions
+  returned by BitstreamWriter.getpos()
+  it has no methods or attributes and can't even be instantiated directly*/
 typedef struct {
     PyObject_HEAD
 
-    BitstreamWriter* bitstream;
-    write_object_f write_unsigned;
-    write_object_f write_signed;
-} bitstream_BitstreamAccumulator;
+    bw_pos_t *pos;
+} bitstream_BitstreamWriterPosition;
 
 static PyObject*
-BitstreamAccumulator_write(bitstream_BitstreamAccumulator *self,
-                           PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_write_signed(bitstream_BitstreamAccumulator *self,
-                                  PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_unary(bitstream_BitstreamAccumulator *self,
-                           PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_write_huffman_code(bitstream_BitstreamAccumulator *self,
-                                        PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_byte_align(bitstream_BitstreamAccumulator *self,
-                                PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_byte_aligned(bitstream_BitstreamAccumulator *self,
-                                  PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_set_endianness(bitstream_BitstreamAccumulator *self,
-                                    PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_write_bytes(bitstream_BitstreamAccumulator *self,
-                                 PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_build(bitstream_BitstreamAccumulator *self,
-                           PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_flush(bitstream_BitstreamAccumulator *self,
-                           PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_close(bitstream_BitstreamAccumulator *self, PyObject *args);
+BitstreamWriterPosition_new(PyTypeObject *type, PyObject *args,
+                            PyObject *kwds);
 
 int
-BitstreamAccumulator_init(bitstream_BitstreamAccumulator *self, PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_bits(bitstream_BitstreamAccumulator *self,
-                          PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_bytes(bitstream_BitstreamAccumulator *self,
-                           PyObject *args);
-
-static PyObject*
-BitstreamAccumulator_reset(bitstream_BitstreamAccumulator *self,
-                           PyObject *args);
-
-
-PyMethodDef BitstreamAccumulator_methods[] = {
-    {"write", (PyCFunction)BitstreamAccumulator_write, METH_VARARGS,
-     "write(bits, unsigned int)"},
-    {"write_signed", (PyCFunction)BitstreamAccumulator_write_signed,
-     METH_VARARGS,
-    "write_signed(bits, signed int)"},
-    {"unary", (PyCFunction)BitstreamAccumulator_unary, METH_VARARGS,
-     "unary(stop_bit, unsigned int)\n"
-     "where \"stop_bit\" must be 0 or 1\n"
-     "writes value as the given number of not stop_bit values (1 or 0)\n"
-     "followed by a stop_bit"},
-     {"write_huffman_code",
-     (PyCFunction)BitstreamAccumulator_write_huffman_code,
-     METH_VARARGS, "write_huffman_code(huffman_tree, value)\n"
-     "given a compiled HuffmanTree and int value,\n"
-     "writes that value to the stream"},
-    {"byte_align", (PyCFunction)BitstreamAccumulator_byte_align, METH_NOARGS,
-     "byte_align()\n"
-     "pads the stream with 0 bits until the next whole byte"},
-    {"byte_aligned", (PyCFunction)BitstreamAccumulator_byte_aligned,
-     METH_NOARGS,
-     "byte_aligned() -> True if the stream is currently byte-aligned"},
-    {"flush", (PyCFunction)BitstreamAccumulator_flush, METH_NOARGS,
-     "flush()\n"
-     "flushes pending data to any underlying file object"},
-    {"close", (PyCFunction)BitstreamAccumulator_close, METH_NOARGS,
-     "close()\n"
-     "closes the stream and any underlying file object"},
-    {"set_endianness", (PyCFunction)BitstreamAccumulator_set_endianness,
-     METH_VARARGS,
-     "set_endianness(endianness)\n"
-     "where 0 = big endian, 1 = little endian"},
-    {"write_bytes", (PyCFunction)BitstreamAccumulator_write_bytes, METH_VARARGS,
-     "write_bytes(bytes, string)"},
-    {"build", (PyCFunction)BitstreamAccumulator_build, METH_VARARGS,
-     "build(format_string, [value1, value2, ...])\n"
-     "where \"format_string\" maps to the calls:\n"
-     "\"#u\" -> write(#, unsigned int value)\n"
-     "\"#s\" -> write_signed(#, signed int value)\n"
-     "\"#p\" -> write(#, 0)\n"
-     "\"#P\" -> write(# * 8, 0)\n"
-     "\"#b\" -> write_bytes(#, string value)\n"
-     "\"a\"  -> byte_align()\n\n"
-     "for instance:\n"
-     "a.build(\"3u 4s 36\", [1, -2, 3L])\n   ==\n"
-     "a.write(3, 1); a.write_signed(4, -2); a.write(36, 3L)"},
-    {"bits", (PyCFunction)BitstreamAccumulator_bits, METH_NOARGS,
-     "bits() -> unsigned int\n"
-     "returns the total number of bits written thus far"},
-    {"bytes", (PyCFunction)BitstreamAccumulator_bytes, METH_NOARGS,
-     "bytes() -> unsigned int\n"
-     "returns the total number of bytes written thus far"},
-    {"reset", (PyCFunction)BitstreamAccumulator_reset, METH_NOARGS,
-     "reset()\n"},
-    {NULL}
-};
+BitstreamWriterPosition_init(bitstream_BitstreamWriterPosition *self,
+                             PyObject *args);
 
 void
-BitstreamAccumulator_dealloc(bitstream_BitstreamAccumulator *self);
+BitstreamWriterPosition_dealloc(bitstream_BitstreamWriterPosition *self);
 
-static PyObject*
-BitstreamAccumulator_new(PyTypeObject *type, PyObject *args,
-                         PyObject *kwds);
-
-PyTypeObject bitstream_BitstreamAccumulatorType = {
+PyTypeObject bitstream_BitstreamWriterPositionType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "bitstream.BitstreamAccumulator",    /*tp_name*/
-    sizeof(bitstream_BitstreamAccumulator), /*tp_basicsize*/
+    "bitstream.BitstreamWriterPosition",    /*tp_name*/
+    sizeof(bitstream_BitstreamWriterPosition), /*tp_basicsize*/
     0,                         /*tp_itemsize*/
-    (destructor)BitstreamAccumulator_dealloc, /*tp_dealloc*/
+    (destructor)BitstreamWriterPosition_dealloc, /*tp_dealloc*/
     0,                         /*tp_print*/
     0,                         /*tp_getattr*/
     0,                         /*tp_setattr*/
@@ -1045,14 +948,14 @@ PyTypeObject bitstream_BitstreamAccumulatorType = {
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
-    "BitstreamAccumulator objects", /* tp_doc */
+    "BitstreamWriterPosition", /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
-    0,                         /*  tp_weaklistoffset */
+    0,                         /* tp_weaklistoffset */
     0,                         /* tp_iter */
     0,                         /* tp_iternext */
-    BitstreamAccumulator_methods,   /* tp_methods */
+    0,                         /* tp_methods */
     0,                         /* tp_members */
     0,                         /* tp_getset */
     0,                         /* tp_base */
@@ -1060,16 +963,26 @@ PyTypeObject bitstream_BitstreamAccumulatorType = {
     0,                         /* tp_descr_get */
     0,                         /* tp_descr_set */
     0,                         /* tp_dictoffset */
-    (initproc)BitstreamAccumulator_init,/* tp_init */
+    (initproc)BitstreamWriterPosition_init, /* tp_init */
     0,                         /* tp_alloc */
-    BitstreamAccumulator_new,  /* tp_new */
+    BitstreamWriterPosition_new, /* tp_new */
 };
 
+/*given a BitstreamReader, format string and list object
+  parses the format from the reader and appends Python values to the list
+  returns 0 on success, 1 on failure (with PyErr set)*/
+int
+bitstream_parse(BitstreamReader* stream,
+                const char* format,
+                PyObject* values);
+
+/*given a BitstreamWriter, format string and PySequence of Python values,
+  writes those values to the writer
+  returns 0 on success, 1 on failure (with PyErr set)*/
+int
+bitstream_build(BitstreamWriter* stream,
+                const char* format,
+                PyObject* iterator);
+
 void
 BitstreamWriter_callback(uint8_t byte, PyObject *callback);
-
-void
-br_close_internal_stream_python_file(BitstreamReader* bs);
-
-void
-bw_close_internal_stream_python_file(BitstreamWriter* bs);
