@@ -1,6 +1,6 @@
 /********************************************************
  Audio Tools, a module and set of tools for manipulating audio data
- Copyright (C) 2007-2014  Brian Langenberger
+ Copyright (C) 2007-2015  Brian Langenberger
 
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
@@ -24,9 +24,7 @@ PyMethodDef module_methods[] = {
 typedef struct {
     PyObject_HEAD
 
-    struct pcmreader_s* pcmreader;
-    aa_int* input_channels;
-    a_int* output_channel;
+    struct PCMReader *pcmreader;
     PyObject* audiotools_pcm;
 } pcmconverter_Averager;
 
@@ -51,12 +49,12 @@ Averager_close(pcmconverter_Averager *self, PyObject *args);
 static PyObject*
 Averager_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
-void
-Averager_dealloc(pcmconverter_Averager *self);
-
 int
 Averager_init(pcmconverter_Averager *self, PyObject *args, PyObject *kwds);
 
+void
+Averager_dealloc(pcmconverter_Averager *self);
+
 PyGetSetDef Averager_getseters[] = {
     {"sample_rate", (getter)Averager_sample_rate, NULL, "sample rate", NULL},
     {"bits_per_sample", (getter)Averager_bits_per_sample, NULL, "bits per sample", NULL},
@@ -73,7 +71,7 @@ PyMethodDef Averager_methods[] = {
 
 PyTypeObject pcmconverter_AveragerType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "pcmconverter.Averager",     /*tp_name*/
+    "pcmconverter.Averager",   /*tp_name*/
     sizeof(pcmconverter_Averager),/*tp_basicsize*/
     0,                         /*tp_itemsize*/
     (destructor)Averager_dealloc, /*tp_dealloc*/
@@ -116,11 +114,7 @@ PyTypeObject pcmconverter_AveragerType = {
 typedef struct {
     PyObject_HEAD
 
-    struct pcmreader_s* pcmreader;
-    aa_int* input_channels;
-    a_int* empty_channel;
-    al_int* six_channels;
-    aa_int* output_channels;
+    struct PCMReader *pcmreader;
     PyObject* audiotools_pcm;
 } pcmconverter_Downmixer;
 
@@ -145,12 +139,12 @@ Downmixer_close(pcmconverter_Downmixer *self, PyObject *args);
 static PyObject*
 Downmixer_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
-void
-Downmixer_dealloc(pcmconverter_Downmixer *self);
-
 int
 Downmixer_init(pcmconverter_Downmixer *self, PyObject *args, PyObject *kwds);
 
+void
+Downmixer_dealloc(pcmconverter_Downmixer *self);
+
 PyGetSetDef Downmixer_getseters[] = {
     {"sample_rate", (getter)Downmixer_sample_rate, NULL, "sample rate", NULL},
     {"bits_per_sample", (getter)Downmixer_bits_per_sample, NULL, "bits per sample", NULL},
@@ -167,7 +161,7 @@ PyMethodDef Downmixer_methods[] = {
 
 PyTypeObject pcmconverter_DownmixerType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "pcmconverter.Downmixer",     /*tp_name*/
+    "pcmconverter.Downmixer",  /*tp_name*/
     sizeof(pcmconverter_Downmixer),/*tp_basicsize*/
     0,                         /*tp_itemsize*/
     (destructor)Downmixer_dealloc, /*tp_dealloc*/
@@ -206,54 +200,12 @@ PyTypeObject pcmconverter_DownmixerType = {
     Downmixer_new,             /* tp_new */
 };
 
-struct float_buffer {
-    float *_;
-    unsigned frames;
-    unsigned max_frames;
-    unsigned channels;
-    unsigned quantization;
-    int min_sample;
-    int max_sample;
-};
-
-static struct float_buffer*
-fb_init(unsigned channels,
-        unsigned bits_per_sample,
-        unsigned max_frames);
-
-static void
-fb_free(struct float_buffer *buffer);
-
-/*amount of frames one can place in the buffer's available space*/
-static unsigned
-fb_available_frames(const struct float_buffer *buffer);
-
-/*increases the frames the buffer can hold by the given amount*/
-static void
-fb_increase_frame_size(struct float_buffer *buffer, unsigned frames);
-
-/*converts each channel in "samples" to floats and appends them to buffer*/
-static void
-fb_append_samples(struct float_buffer *buffer, const aa_int *samples);
-
-/*removes the given number of frames from the start of the buffer*/
-static void
-fb_pop_frames(struct float_buffer *buffer, unsigned frames);
-
-/*quantizes frames in buffer and appends them to samples*/
-static void
-fb_export_frames(struct float_buffer *buffer, a_int *samples);
-
 typedef struct {
     PyObject_HEAD
 
-    struct pcmreader_s* pcmreader;
-    aa_int *pcmreader_channels;      /*a given read() call's input buffer*/
+    struct PCMReader *pcmreader;
     SRC_STATE *src_state;            /*libsamplerate's internal state*/
-    double ratio;                    /*the conversion ratio*/
-    struct float_buffer *in_buffer;  /*input floating point samples*/
-    struct float_buffer *out_buffer; /*output floating point samples*/
-    a_int *output_framelist;         /*a FrameList output buffer*/
+    SRC_DATA src_data;               /*libsamplerate's processing state*/
     int sample_rate;                 /*the output sample rate*/
     PyObject* audiotools_pcm;
 } pcmconverter_Resampler;
@@ -279,12 +231,12 @@ Resampler_close(pcmconverter_Resampler *self, PyObject *args);
 static PyObject*
 Resampler_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
-void
-Resampler_dealloc(pcmconverter_Resampler *self);
-
 int
 Resampler_init(pcmconverter_Resampler *self, PyObject *args, PyObject *kwds);
 
+void
+Resampler_dealloc(pcmconverter_Resampler *self);
+
 PyGetSetDef Resampler_getseters[] = {
     {"sample_rate", (getter)Resampler_sample_rate, NULL, "sample rate", NULL},
     {"bits_per_sample", (getter)Resampler_bits_per_sample, NULL, "bits per sample", NULL},
@@ -301,7 +253,7 @@ PyMethodDef Resampler_methods[] = {
 
 PyTypeObject pcmconverter_ResamplerType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "pcmconverter.Resampler",     /*tp_name*/
+    "pcmconverter.Resampler",  /*tp_name*/
     sizeof(pcmconverter_Resampler),/*tp_basicsize*/
     0,                         /*tp_itemsize*/
     (destructor)Resampler_dealloc, /*tp_dealloc*/
@@ -343,12 +295,10 @@ PyTypeObject pcmconverter_ResamplerType = {
 typedef struct {
     PyObject_HEAD
 
-    struct pcmreader_s* pcmreader;
+    struct PCMReader *pcmreader;
     int bits_per_sample;
-    aa_int* input_channels;
-    aa_int* output_channels;
-    BitstreamReader* white_noise;
-    PyObject* audiotools_pcm;
+    BitstreamReader *white_noise;
+    PyObject *audiotools_pcm;
 } pcmconverter_BPSConverter;
 
 static PyObject*
@@ -372,13 +322,13 @@ BPSConverter_close(pcmconverter_BPSConverter *self, PyObject *args);
 static PyObject*
 BPSConverter_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
 
-void
-BPSConverter_dealloc(pcmconverter_BPSConverter *self);
-
 int
 BPSConverter_init(pcmconverter_BPSConverter *self,
                   PyObject *args, PyObject *kwds);
 
+void
+BPSConverter_dealloc(pcmconverter_BPSConverter *self);
+
 PyGetSetDef BPSConverter_getseters[] = {
     {"sample_rate", (getter)BPSConverter_sample_rate,
      NULL, "sample rate", NULL},
@@ -399,7 +349,7 @@ PyMethodDef BPSConverter_methods[] = {
 
 PyTypeObject pcmconverter_BPSConverterType = {
     PyVarObject_HEAD_INIT(NULL, 0)
-    "pcmconverter.BPSConverter",     /*tp_name*/
+    "pcmconverter.BPSConverter", /*tp_name*/
     sizeof(pcmconverter_BPSConverter),/*tp_basicsize*/
     0,                         /*tp_itemsize*/
     (destructor)BPSConverter_dealloc, /*tp_dealloc*/
@@ -418,22 +368,340 @@ PyTypeObject pcmconverter_BPSConverterType = {
     0,                         /*tp_setattro*/
     0,                         /*tp_as_buffer*/
     Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
-    "BPSConverter objects",       /* tp_doc */
+    "BPSConverter objects",    /* tp_doc */
     0,                         /* tp_traverse */
     0,                         /* tp_clear */
     0,                         /* tp_richcompare */
     0,                         /* tp_weaklistoffset */
     0,                         /* tp_iter */
     0,                         /* tp_iternext */
-    BPSConverter_methods,         /* tp_methods */
+    BPSConverter_methods,      /* tp_methods */
     0,                         /* tp_members */
-    BPSConverter_getseters,       /* tp_getset */
+    BPSConverter_getseters,    /* tp_getset */
     0,                         /* tp_base */
     0,                         /* tp_dict */
     0,                         /* tp_descr_get */
     0,                         /* tp_descr_set */
     0,                         /* tp_dictoffset */
-    (initproc)BPSConverter_init,  /* tp_init */
+    (initproc)BPSConverter_init, /* tp_init */
     0,                         /* tp_alloc */
-    BPSConverter_new,             /* tp_new */
+    BPSConverter_new,          /* tp_new */
+};
+
+
+typedef struct {
+    PyObject_HEAD
+
+    int closed;
+    struct PCMReader *pcmreader;
+    PyObject *audiotools_pcm;
+} pcmconverter_BufferedPCMReader;
+
+static PyObject*
+BufferedPCMReader_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
+
+int
+BufferedPCMReader_init(pcmconverter_BufferedPCMReader *self,
+                       PyObject *args, PyObject *kwds);
+
+void
+BufferedPCMReader_dealloc(pcmconverter_BufferedPCMReader *self);
+
+static PyObject*
+BufferedPCMReader_sample_rate(pcmconverter_BufferedPCMReader *self,
+                              void *closure);
+
+static PyObject*
+BufferedPCMReader_bits_per_sample(pcmconverter_BufferedPCMReader *self,
+                                  void *closure);
+
+static PyObject*
+BufferedPCMReader_channels(pcmconverter_BufferedPCMReader *self,
+                           void *closure);
+
+static PyObject*
+BufferedPCMReader_channel_mask(pcmconverter_BufferedPCMReader *self,
+                               void *closure);
+
+static PyObject*
+BufferedPCMReader_read(pcmconverter_BufferedPCMReader *self, PyObject *args);
+
+static PyObject*
+BufferedPCMReader_close(pcmconverter_BufferedPCMReader *self, PyObject *args);
+
+static PyObject*
+BufferedPCMReader_enter(pcmconverter_BufferedPCMReader *self, PyObject *args);
+
+static PyObject*
+BufferedPCMReader_exit(pcmconverter_BufferedPCMReader *self, PyObject *args);
+
+PyGetSetDef BufferedPCMReader_getseters[] = {
+    {"sample_rate", (getter)BufferedPCMReader_sample_rate,
+     NULL, "sample rate", NULL},
+    {"bits_per_sample", (getter)BufferedPCMReader_bits_per_sample,
+     NULL, "bits per sample", NULL},
+    {"channels", (getter)BufferedPCMReader_channels,
+     NULL, "channels", NULL},
+    {"channel_mask", (getter)BufferedPCMReader_channel_mask,
+     NULL, "channel_mask", NULL},
+    {NULL}
+};
+
+PyMethodDef BufferedPCMReader_methods[] = {
+    {"read", (PyCFunction)BufferedPCMReader_read, METH_VARARGS, ""},
+    {"close", (PyCFunction)BufferedPCMReader_close, METH_NOARGS, ""},
+    {"__enter__", (PyCFunction)BufferedPCMReader_enter,
+     METH_NOARGS, "enter() -> self"},
+    {"__exit__", (PyCFunction)BufferedPCMReader_exit,
+     METH_VARARGS, "exit(exc_type, exc_value, traceback) -> None"},
+    {NULL}
+};
+
+PyTypeObject pcmconverter_BufferedPCMReaderType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "pcmconverter.BufferedPCMReader", /*tp_name*/
+    sizeof(pcmconverter_BufferedPCMReader), /*tp_basicsize*/
+    0,                         /*tp_itemsize*/
+    (destructor)BufferedPCMReader_dealloc, /*tp_dealloc*/
+    0,                         /*tp_print*/
+    0,                         /*tp_getattr*/
+    0,                         /*tp_setattr*/
+    0,                         /*tp_compare*/
+    0,                         /*tp_repr*/
+    0,                         /*tp_as_number*/
+    0,                         /*tp_as_sequence*/
+    0,                         /*tp_as_mapping*/
+    0,                         /*tp_hash */
+    0,                         /*tp_call*/
+    0,                         /*tp_str*/
+    0,                         /*tp_getattro*/
+    0,                         /*tp_setattro*/
+    0,                         /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
+    "BufferedPCMReader objects", /* tp_doc */
+    0,                         /* tp_traverse */
+    0,                         /* tp_clear */
+    0,                         /* tp_richcompare */
+    0,                         /* tp_weaklistoffset */
+    0,                         /* tp_iter */
+    0,                         /* tp_iternext */
+    BufferedPCMReader_methods, /* tp_methods */
+    0,                         /* tp_members */
+    BufferedPCMReader_getseters, /* tp_getset */
+    0,                         /* tp_base */
+    0,                         /* tp_dict */
+    0,                         /* tp_descr_get */
+    0,                         /* tp_descr_set */
+    0,                         /* tp_dictoffset */
+    (initproc)BufferedPCMReader_init, /* tp_init */
+    0,                         /* tp_alloc */
+    BufferedPCMReader_new,     /* tp_new */
+};
+
+
+typedef struct {
+    PyObject_HEAD
+
+    int closed;
+    struct PCMReader *pcmreader;
+
+    unsigned frame_index;
+    unsigned frame_total;
+
+    PyObject *audiotools_pcm;
+} pcmconverter_FadeInReader;
+
+static PyObject*
+FadeInReader_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
+
+int
+FadeInReader_init(pcmconverter_FadeInReader *self,
+                  PyObject *args, PyObject *kwds);
+
+void
+FadeInReader_dealloc(pcmconverter_FadeInReader *self);
+
+static PyObject*
+FadeInReader_sample_rate(pcmconverter_FadeInReader *self,
+                         void *closure);
+
+static PyObject*
+FadeInReader_bits_per_sample(pcmconverter_FadeInReader *self,
+                             void *closure);
+
+static PyObject*
+FadeInReader_channels(pcmconverter_FadeInReader *self,
+                      void *closure);
+
+static PyObject*
+FadeInReader_channel_mask(pcmconverter_FadeInReader *self,
+                          void *closure);
+
+static PyObject*
+FadeInReader_read(pcmconverter_FadeInReader *self, PyObject *args);
+
+static PyObject*
+FadeInReader_close(pcmconverter_FadeInReader *self, PyObject *args);
+
+PyGetSetDef FadeInReader_getseters[] = {
+    {"sample_rate", (getter)FadeInReader_sample_rate,
+     NULL, "sample rate", NULL},
+    {"bits_per_sample", (getter)FadeInReader_bits_per_sample,
+     NULL, "bits per sample", NULL},
+    {"channels", (getter)FadeInReader_channels,
+     NULL, "channels", NULL},
+    {"channel_mask", (getter)FadeInReader_channel_mask,
+     NULL, "channel_mask", NULL},
+    {NULL}
+};
+
+PyMethodDef FadeInReader_methods[] = {
+    {"read", (PyCFunction)FadeInReader_read, METH_VARARGS, ""},
+    {"close", (PyCFunction)FadeInReader_close, METH_NOARGS, ""},
+    {NULL}
+};
+
+PyTypeObject pcmconverter_FadeInReaderType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "pcmconverter.FadeInReader", /*tp_name*/
+    sizeof(pcmconverter_FadeInReader), /*tp_basicsize*/
+    0,                         /*tp_itemsize*/
+    (destructor)FadeInReader_dealloc, /*tp_dealloc*/
+    0,                         /*tp_print*/
+    0,                         /*tp_getattr*/
+    0,                         /*tp_setattr*/
+    0,                         /*tp_compare*/
+    0,                         /*tp_repr*/
+    0,                         /*tp_as_number*/
+    0,                         /*tp_as_sequence*/
+    0,                         /*tp_as_mapping*/
+    0,                         /*tp_hash */
+    0,                         /*tp_call*/
+    0,                         /*tp_str*/
+    0,                         /*tp_getattro*/
+    0,                         /*tp_setattro*/
+    0,                         /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
+    "FadeInReader objects", /* tp_doc */
+    0,                         /* tp_traverse */
+    0,                         /* tp_clear */
+    0,                         /* tp_richcompare */
+    0,                         /* tp_weaklistoffset */
+    0,                         /* tp_iter */
+    0,                         /* tp_iternext */
+    FadeInReader_methods,      /* tp_methods */
+    0,                         /* tp_members */
+    FadeInReader_getseters,    /* tp_getset */
+    0,                         /* tp_base */
+    0,                         /* tp_dict */
+    0,                         /* tp_descr_get */
+    0,                         /* tp_descr_set */
+    0,                         /* tp_dictoffset */
+    (initproc)FadeInReader_init, /* tp_init */
+    0,                         /* tp_alloc */
+    FadeInReader_new,          /* tp_new */
+};
+
+
+typedef struct {
+    PyObject_HEAD
+
+    int closed;
+    struct PCMReader *pcmreader;
+
+    unsigned frame_index;
+    unsigned frame_total;
+
+    PyObject *audiotools_pcm;
+} pcmconverter_FadeOutReader;
+
+static PyObject*
+FadeOutReader_new(PyTypeObject *type, PyObject *args, PyObject *kwds);
+
+int
+FadeOutReader_init(pcmconverter_FadeOutReader *self,
+                   PyObject *args, PyObject *kwds);
+
+void
+FadeOutReader_dealloc(pcmconverter_FadeOutReader *self);
+
+static PyObject*
+FadeOutReader_sample_rate(pcmconverter_FadeOutReader *self,
+                          void *closure);
+
+static PyObject*
+FadeOutReader_bits_per_sample(pcmconverter_FadeOutReader *self,
+                              void *closure);
+
+static PyObject*
+FadeOutReader_channels(pcmconverter_FadeOutReader *self,
+                       void *closure);
+
+static PyObject*
+FadeOutReader_channel_mask(pcmconverter_FadeOutReader *self,
+                           void *closure);
+
+static PyObject*
+FadeOutReader_read(pcmconverter_FadeOutReader *self, PyObject *args);
+
+static PyObject*
+FadeOutReader_close(pcmconverter_FadeOutReader *self, PyObject *args);
+
+PyGetSetDef FadeOutReader_getseters[] = {
+    {"sample_rate", (getter)FadeOutReader_sample_rate,
+     NULL, "sample rate", NULL},
+    {"bits_per_sample", (getter)FadeOutReader_bits_per_sample,
+     NULL, "bits per sample", NULL},
+    {"channels", (getter)FadeOutReader_channels,
+     NULL, "channels", NULL},
+    {"channel_mask", (getter)FadeOutReader_channel_mask,
+     NULL, "channel_mask", NULL},
+    {NULL}
+};
+
+PyMethodDef FadeOutReader_methods[] = {
+    {"read", (PyCFunction)FadeOutReader_read, METH_VARARGS, ""},
+    {"close", (PyCFunction)FadeOutReader_close, METH_NOARGS, ""},
+    {NULL}
+};
+
+PyTypeObject pcmconverter_FadeOutReaderType = {
+    PyVarObject_HEAD_INIT(NULL, 0)
+    "pcmconverter.FadeOutReader", /*tp_name*/
+    sizeof(pcmconverter_FadeOutReader), /*tp_basicsize*/
+    0,                         /*tp_itemsize*/
+    (destructor)FadeOutReader_dealloc, /*tp_dealloc*/
+    0,                         /*tp_print*/
+    0,                         /*tp_getattr*/
+    0,                         /*tp_setattr*/
+    0,                         /*tp_compare*/
+    0,                         /*tp_repr*/
+    0,                         /*tp_as_number*/
+    0,                         /*tp_as_sequence*/
+    0,                         /*tp_as_mapping*/
+    0,                         /*tp_hash */
+    0,                         /*tp_call*/
+    0,                         /*tp_str*/
+    0,                         /*tp_getattro*/
+    0,                         /*tp_setattro*/
+    0,                         /*tp_as_buffer*/
+    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
+    "FadeOutReader objects",   /* tp_doc */
+    0,                         /* tp_traverse */
+    0,                         /* tp_clear */
+    0,                         /* tp_richcompare */
+    0,                         /* tp_weaklistoffset */
+    0,                         /* tp_iter */
+    0,                         /* tp_iternext */
+    FadeOutReader_methods,     /* tp_methods */
+    0,                         /* tp_members */
+    FadeOutReader_getseters,   /* tp_getset */
+    0,                         /* tp_base */
+    0,                         /* tp_dict */
+    0,                         /* tp_descr_get */
+    0,                         /* tp_descr_set */
+    0,                         /* tp_dictoffset */
+    (initproc)FadeOutReader_init, /* tp_init */
+    0,                         /* tp_alloc */
+    FadeOutReader_new,         /* tp_new */
 };
