@@ -1,9 +1,8 @@
 #include <Python.h>
 #include "mod_defs.h"
-#include "pcm.h"
-#include "pcmconv.h"
+#include "framelist.h"
+#include "pcmreader.h"
 #include "bitstream.h"
-#include "array.h"
 #include "dither.c"
 #include "replaygain.h"
 
@@ -46,11 +45,25 @@ PyMethodDef module_methods[] = {
     {NULL}
 };
 
+PyGetSetDef ReplayGain_getseters[] = {
+    {"sample_rate",
+     (getter)ReplayGain_sample_rate, NULL, "sample rate", NULL},
+    {NULL}
+};
+
 PyMethodDef ReplayGain_methods[] = {
-    {"title_gain",(PyCFunction)ReplayGain_title_gain,
-     METH_VARARGS,"PCMReader -> (title gain, title peak) tuple"},
-    {"album_gain",(PyCFunction)ReplayGain_album_gain,
-     METH_NOARGS,"Returns an (album gain, album peak) tuple"},
+    {"update", (PyCFunction)ReplayGain_update,
+     METH_VARARGS, "update(FrameList) -> None"},
+    {"title_gain", (PyCFunction)ReplayGain_title_gain,
+     METH_NOARGS, "title_gain() -> title gain float"},
+    {"title_peak", (PyCFunction)ReplayGain_title_peak,
+     METH_NOARGS, "title_peak() -> title peak float"},
+    {"album_gain", (PyCFunction)ReplayGain_album_gain,
+     METH_NOARGS, "album_gain() -> album gain float"},
+    {"album_peak", (PyCFunction)ReplayGain_album_peak,
+     METH_NOARGS, "album_peak() -> album peak float"},
+    {"next_title", (PyCFunction)ReplayGain_next_title,
+     METH_NOARGS, "call after each title is completed"},
     {NULL}
 };
 
@@ -84,7 +97,7 @@ PyTypeObject replaygain_ReplayGainType = {
     0,                         /* tp_iternext */
     ReplayGain_methods,        /* tp_methods */
     0,                         /* tp_members */
-    0,                         /* tp_getset */
+    ReplayGain_getseters,      /* tp_getset */
     0,                         /* tp_base */
     0,                         /* tp_dict */
     0,                         /* tp_descr_get */
@@ -98,6 +111,7 @@ PyTypeObject replaygain_ReplayGainType = {
 void
 ReplayGain_dealloc(replaygain_ReplayGain* self)
 {
+    Py_XDECREF(self->framelist_type);
     Py_TYPE(self)->tp_free((PyObject*)self);
 }
 
@@ -115,13 +129,26 @@ int
 ReplayGain_init(replaygain_ReplayGain *self, PyObject *args, PyObject *kwds)
 {
     long sample_rate;
+    PyObject *audiotools_pcm;
     int  i;
 
+    self->framelist_type = NULL;
     self->sample_rate = 0;
+    self->title_peak = 0.0;
+    self->album_peak = 0.0;
 
     if (!PyArg_ParseTuple(args, "l", &sample_rate))
         return -1;
 
+    /*store FrameList type for later comparison*/
+    if ((audiotools_pcm = PyImport_ImportModule("audiotools.pcm")) != NULL) {
+        self->framelist_type = PyObject_GetAttrString(audiotools_pcm,
+                                                      "FrameList");
+        Py_DECREF(audiotools_pcm);
+    } else {
+        return -1;
+    }
+
     self->sample_rate = (unsigned)sample_rate;
 
     /* zero out initial values*/
@@ -176,157 +203,145 @@ ReplayGain_init(replaygain_ReplayGain *self, PyObject *args, PyObject *kwds)
 
     memset (self->B, 0, sizeof(self->B));
 
-    self->album_peak = 0.0;
 
     return 0;
 }
 
-
 PyObject*
-ReplayGain_title_gain(replaygain_ReplayGain *self, PyObject *args)
+ReplayGain_sample_rate(replaygain_ReplayGain *self, void *closure)
 {
-    double title_gain;
-    double title_peak = 0.0;
-    pcmreader* pcmreader = NULL;
-
-    /*read PCMReader-compatible object from args*/
-    if (!PyArg_ParseTuple(args, "O&", pcmreader_converter, &pcmreader)) {
-        return NULL;
-    } else {
-        aa_int* channels = aa_int_new();
-        aa_double* channels_f = aa_double_new();
-        const int32_t peak_shift = 1 << (pcmreader->bits_per_sample - 1);
-
-        if (pcmreader->sample_rate != self->sample_rate) {
-            PyErr_SetString(PyExc_ValueError,
-                            "pcmreader's sample rate doesn't match");
-            pcmreader->del(pcmreader);
-            channels->del(channels);
-            channels_f->del(channels_f);
-            return NULL;
-        }
-
-        /*read a FrameList object from PCMReader*/
-        if (pcmreader->read(pcmreader, 4096, channels)) {
-            pcmreader->del(pcmreader);
-            channels->del(channels);
-            channels_f->del(channels_f);
-            return NULL;
-        }
-
-        /*while FrameList contains more samples*/
-        while (channels->_[0]->len) {
-            unsigned c;
-
-            /*ensure FrameList only contains 1 or 2 channels*/
-            if ((channels->len != 1) && (channels->len != 2)) {
-                PyErr_SetString(PyExc_ValueError,
-                                "FrameList must contain only 1 or 2 channels");
-                pcmreader->del(pcmreader);
-                channels->del(channels);
-                channels_f->del(channels_f);
-                return NULL;
-            }
-
-            /*if only one channel, duplicate it to the other channel*/
-            channels->_[0]->copy(channels->_[0], channels->append(channels));
-            channels_f->reset(channels_f);
-
-            /*convert left and right channels to doubles,
-              (but *not* doubles between -1.0 and 1.0)
-              and store peak values*/
-            for (c = 0; c < 2; c++) {
-                a_int* channel_i = channels->_[c];
-                a_double* channel_f = channels_f->append(channels_f);
-                int i;
-                double peak;
-
-                channel_f->resize(channel_f, channel_i->len);
-
-                switch (pcmreader->bits_per_sample) {
-                case 8:
-                    for (i = 0; i < channel_i->len; i++) {
-                        a_append(channel_f, (double)(channel_i->_[i] << 8));
-
-                        peak = ((double)(abs(channel_i->_[i])) / peak_shift);
-                        title_peak = MAX(title_peak, peak);
-                        self->album_peak = MAX(self->album_peak, peak);
-                    }
-                    break;
-                case 16:
-                    for (i = 0; i < channel_i->len; i++) {
-                        a_append(channel_f, (double)(channel_i->_[i]));
-
-                        peak = ((double)(abs(channel_i->_[i])) / peak_shift);
-                        title_peak = MAX(title_peak, peak);
-                        self->album_peak = MAX(self->album_peak, peak);
-                    }
-                    break;
-                case 24:
-                    for (i = 0; i < channel_i->len; i++) {
-                        a_append(channel_f, (double)(channel_i->_[i] >> 8));
-
-                        peak = ((double)(abs(channel_i->_[i])) / peak_shift);
-                        title_peak = MAX(title_peak, peak);
-                        self->album_peak = MAX(self->album_peak, peak);
-                    }
-                    break;
-                default:
-                    PyErr_SetString(PyExc_ValueError,
-                                    "unsupported bits per sample");
-                    pcmreader->del(pcmreader);
-                    channels->del(channels);
-                    channels_f->del(channels_f);
-                    return NULL;
-                }
-            }
-
-            /*perform actual gain analysis on channels*/
-            if (ReplayGain_analyze_samples(self,
-                                           channels_f->_[0]->_,
-                                           channels_f->_[1]->_,
-                                           channels_f->_[0]->len,
-                                           2) == GAIN_ANALYSIS_ERROR) {
-                PyErr_SetString(PyExc_ValueError,
-                                "ReplayGain calculation error");
-                pcmreader->del(pcmreader);
-                channels->del(channels);
-                channels_f->del(channels_f);
-                return NULL;
-            }
-
-            /*read next FrameList object from PCMReader*/
-            if (pcmreader->read(pcmreader, 4096, channels)) {
-                pcmreader->del(pcmreader);
-                channels->del(channels);
-                channels_f->del(channels_f);
-                return NULL;
-            }
-        }
-
-        /*deallocate temporary variables*/
-        pcmreader->del(pcmreader);
-        channels->del(channels);
-        channels_f->del(channels_f);
-
-        /*return calculated title gain and title peak*/
-        /*if enough samples have been read*/
-        if ((title_gain = ReplayGain_get_title_gain(self)) !=
-            GAIN_NOT_ENOUGH_SAMPLES) {
-            return Py_BuildValue("(d,d)", title_gain, title_peak);
-        } else {
-            title_gain = 0.0;
-            return Py_BuildValue("(d,d)", title_gain, title_peak);
-        }
-    }
+    return Py_BuildValue("I", self->sample_rate);
 }
 
 PyObject*
-ReplayGain_album_gain(replaygain_ReplayGain *self)
+ReplayGain_next_title(replaygain_ReplayGain *self)
 {
-    double gain_value = ReplayGain_get_album_gain(self);
+    /*reset initial values for next title*/
+    int    i;
+
+    for ( i = 0; i < (int)(sizeof(self->A)/sizeof(*(self->A))); i++ ) {
+        self->B[i] += self->A[i];
+        self->A[i]  = 0;
+    }
+
+    for ( i = 0; i < MAX_ORDER; i++ )
+        self->linprebuf[i] =
+            self->lstepbuf[i] =
+            self->loutbuf[i] =
+            self->rinprebuf[i] =
+            self->rstepbuf[i] =
+            self->routbuf[i] = 0.f;
+
+    self->totsamp = 0;
+    self->lsum    = self->rsum = 0.;
+
+    self->title_peak = 0.0;
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+#define CHUNK_SIZE 4096
+
+PyObject*
+ReplayGain_update(replaygain_ReplayGain *self, PyObject *args)
+{
+    pcm_FrameList* framelist;
+    unsigned total_frames;
+    const int *samples;
+    int32_t peak_shift;
+    static int left_i[CHUNK_SIZE];
+    static int right_i[CHUNK_SIZE];
+    static double left_f[CHUNK_SIZE];
+    static double right_f[CHUNK_SIZE];
+
+    if (!PyArg_ParseTuple(args, "O!", self->framelist_type, &framelist))
+        return NULL;
+
+    peak_shift = 1 << (framelist->bits_per_sample - 1);
+    total_frames = framelist->frames;
+    samples = framelist->samples;
+
+    /*FrameList could be very large, so process it in chunks
+      rather than all at once*/
+    while (total_frames) {
+        const unsigned to_process = MIN(total_frames, CHUNK_SIZE);
+        unsigned i;
+
+        /*split FrameList's packed ints into a set of channels
+          to a maximum of 2 channels*/
+        get_channel_data(samples,
+                         0,
+                         framelist->channels,
+                         to_process,
+                         left_i);
+
+        /*if 1 channel, duplicate to right channel*/
+        get_channel_data(samples,
+                         framelist->channels > 1 ? 1 : 0,
+                         framelist->channels,
+                         to_process,
+                         right_i);
+
+
+        /*calculate peak values*/
+        for (i = 0; i < to_process; i++) {
+            const double peak_l = ((double)(abs(left_i[i]))) / peak_shift;
+            const double peak_r = ((double)(abs(right_i[i]))) / peak_shift;
+            const double peak = MAX(peak_l, peak_r);
+            self->title_peak = MAX(self->title_peak, peak);
+            self->album_peak = MAX(self->album_peak, peak);
+        }
+
+        /*convert channels to 16-bit doubles*/
+        switch (framelist->bits_per_sample) {
+        case 8:
+            for (i = 0; i < to_process; i++) {
+                left_f[i] = (double)(left_i[i] << 8);
+                right_f[i] = (double)(right_i[i] << 8);
+            }
+            break;
+        case 16:
+            for (i = 0; i < to_process; i++) {
+                left_f[i] = (double)(left_i[i]);
+                right_f[i] = (double)(right_i[i]);
+            }
+            break;
+        case 24:
+            for (i = 0; i < to_process; i++) {
+                left_f[i] = (double)(left_i[i] >> 8);
+                right_f[i] = (double)(right_i[i] >> 8);
+            }
+            break;
+        default:
+            PyErr_SetString(PyExc_ValueError, "unsupported bits per sample");
+            return NULL;
+        }
+
+        /*perform gain analysis on channels*/
+        if (ReplayGain_analyze_samples(self,
+                                       left_f,
+                                       right_f,
+                                       to_process,
+                                       2) == GAIN_ANALYSIS_ERROR) {
+            PyErr_SetString(PyExc_ValueError, "ReplayGain calculation error");
+            return NULL;
+        }
+
+        total_frames -= to_process;
+        samples += (to_process * framelist->channels);
+    }
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
+PyObject*
+ReplayGain_title_gain(replaygain_ReplayGain *self)
+{
+    const double gain_value = ReplayGain_get_title_gain(self);
     if (gain_value != GAIN_NOT_ENOUGH_SAMPLES) {
-        return Py_BuildValue("(d,d)",gain_value,self->album_peak);
+        return Py_BuildValue("d", gain_value);
     } else {
         PyErr_SetString(PyExc_ValueError,
                         "Not enough samples to perform calculation");
@@ -334,6 +349,30 @@ ReplayGain_album_gain(replaygain_ReplayGain *self)
     }
 }
 
+PyObject*
+ReplayGain_title_peak(replaygain_ReplayGain *self)
+{
+    return Py_BuildValue("d", self->title_peak);
+}
+
+PyObject*
+ReplayGain_album_gain(replaygain_ReplayGain *self)
+{
+    const double gain_value = ReplayGain_get_album_gain(self);
+    if (gain_value != GAIN_NOT_ENOUGH_SAMPLES) {
+        return Py_BuildValue("d", gain_value);
+    } else {
+        PyErr_SetString(PyExc_ValueError,
+                        "Not enough samples to perform calculation");
+        return NULL;
+    }
+}
+
+PyObject*
+ReplayGain_album_peak(replaygain_ReplayGain *self)
+{
+    return Py_BuildValue("d", self->album_peak);
+}
 
 PyGetSetDef ReplayGainReader_getseters[] = {
     {"sample_rate",
@@ -777,27 +816,7 @@ analyzeResult(uint32_t* Array, size_t len)
 double
 ReplayGain_get_title_gain(replaygain_ReplayGain *self)
 {
-    double  retval;
-    int    i;
-
-    retval = analyzeResult(self->A, sizeof(self->A)/sizeof(*(self->A)) );
-
-    for ( i = 0; i < (int)(sizeof(self->A)/sizeof(*(self->A))); i++ ) {
-        self->B[i] += self->A[i];
-        self->A[i]  = 0;
-    }
-
-    for ( i = 0; i < MAX_ORDER; i++ )
-        self->linprebuf[i] =
-            self->lstepbuf[i] =
-            self->loutbuf[i] =
-            self->rinprebuf[i] =
-            self->rstepbuf[i] =
-            self->routbuf[i] = 0.f;
-
-    self->totsamp = 0;
-    self->lsum    = self->rsum = 0.;
-    return retval;
+    return analyzeResult(self->A, sizeof(self->A)/sizeof(*(self->A)) );
 }
 
 
@@ -820,16 +839,17 @@ ReplayGainReader_new(PyTypeObject *type, PyObject *args, PyObject *kwds) {
 int
 ReplayGainReader_init(replaygain_ReplayGainReader *self,
                       PyObject *args, PyObject *kwds) {
-    self->pcmreader = NULL;
-    self->channels = aa_int_new();
-    self->white_noise = NULL;
-    self->audiotools_pcm = NULL;
-
     double replaygain;
     double peak;
 
+    self->stream_closed = 0;
+    self->pcmreader = NULL;
+    self->white_noise = NULL;
+    self->audiotools_pcm = NULL;
+
+
     if (!PyArg_ParseTuple(args, "O&dd",
-                          pcmreader_converter, &(self->pcmreader),
+                          py_obj_to_pcmreader, &(self->pcmreader),
                           &(replaygain),
                           &(peak)))
         return -1;
@@ -851,7 +871,6 @@ void
 ReplayGainReader_dealloc(replaygain_ReplayGainReader* self) {
     if (self->pcmreader != NULL)
         self->pcmreader->del(self->pcmreader);
-    self->channels = aa_int_new();
     if (self->white_noise != NULL)
         self->white_noise->close(self->white_noise);
     Py_XDECREF(self->audiotools_pcm);
@@ -885,13 +904,12 @@ ReplayGainReader_channel_mask(replaygain_ReplayGainReader *self,
 
 static PyObject*
 ReplayGainReader_read(replaygain_ReplayGainReader* self, PyObject *args) {
-    int pcm_frames = 0;
-    aa_int* channels = self->channels;
-    unsigned c;
+    int pcm_frames;
 
-    const int max_value = (1 << (self->pcmreader->bits_per_sample - 1)) - 1;
-    const int min_value = -(1 << (self->pcmreader->bits_per_sample - 1));
-    const double multiplier = self->multiplier;
+    if (self->stream_closed) {
+        PyErr_SetString(PyExc_ValueError, "unable to read from closed stream");
+        return NULL;
+    }
 
     if (!PyArg_ParseTuple(args, "i", &pcm_frames))
         return NULL;
@@ -899,35 +917,52 @@ ReplayGainReader_read(replaygain_ReplayGainReader* self, PyObject *args) {
     if (pcm_frames <= 0) {
         PyErr_SetString(PyExc_ValueError, "pcm_frames must be positive");
         return NULL;
-    }
+    } else {
+        const int max_value = (1 << (self->pcmreader->bits_per_sample - 1)) - 1;
+        const int min_value = -(1 << (self->pcmreader->bits_per_sample - 1));
+        const double multiplier = self->multiplier;
 
-    /*read FrameList object from internal PCMReader*/
-    if (self->pcmreader->read(self->pcmreader,
-                              (unsigned)pcm_frames,
-                              channels))
-        return NULL;
+        pcm_FrameList *framelist = new_FrameList(
+            self->audiotools_pcm,
+            self->pcmreader->channels,
+            self->pcmreader->bits_per_sample,
+            pcm_frames);
 
-    /*apply our multiplier to framelist's integer samples
-      and apply dithering*/
-    for (c = 0; c < channels->len; c++) {
-        a_int* channel = channels->_[c];
+        const unsigned frames_read =
+            self->pcmreader->read(self->pcmreader,
+                                  pcm_frames,
+                                  framelist->samples);
+        const unsigned total_samples =
+            frames_read * self->pcmreader->channels;
         unsigned i;
-        for (i = 0; i < channel->len; i++) {
-            channel->_[i] = (int)lround(channel->_[i] * multiplier);
-            channel->_[i] = (MIN(MAX(channel->_[i], min_value), max_value) ^
-                             self->white_noise->read(self->white_noise, 1));
-        }
-    }
 
-    /*return integer samples as a new FrameList object*/
-    return aa_int_to_FrameList(self->audiotools_pcm,
-                               channels,
-                               self->pcmreader->bits_per_sample);
+        if (!frames_read && (self->pcmreader->status != PCM_OK)) {
+            Py_DECREF((PyObject*)framelist);
+            return NULL;
+        } else {
+            framelist->frames = frames_read;
+            framelist->samples_length = total_samples;
+        }
+
+        /*apply our multiplier to framelist's integer samples
+          and apply dithering*/
+        for (i = 0; i < total_samples; i++) {
+            framelist->samples[i] =
+                (int)lround(framelist->samples[i] * multiplier);
+            framelist->samples[i] =
+                (MIN(MAX(framelist->samples[i], min_value), max_value) ^
+                 self->white_noise->read(self->white_noise, 1));
+        }
+
+        /*return integer samples as a new FrameList object*/
+        return (PyObject*)framelist;
+    }
 }
 
 static PyObject*
 ReplayGainReader_close(replaygain_ReplayGainReader* self, PyObject *args) {
     self->pcmreader->close(self->pcmreader);
+    self->stream_closed = 1;
     Py_INCREF(Py_None);
     return Py_None;
 }
