@@ -1,27 +1,38 @@
 /********************************************************
- Audio Tools, a module and set of tools for manipulating audio data
- Copyright (C) 2007-2014  Brian Langenberger
+ Bitstream Library, a module for reading bits of data
 
- This program is free software; you can redistribute it and/or modify
- it under the terms of the GNU General Public License as published by
- the Free Software Foundation; either version 2 of the License, or
- (at your option) any later version.
+ Copyright (C) 2007-2015  Brian Langenberger
 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
+ The Bitstream Library is free software; you can redistribute it and/or modify
+ it under the terms of either:
 
- You should have received a copy of the GNU General Public License
- along with this program; if not, write to the Free Software
- Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-*******************************************************/
+   * the GNU Lesser General Public License as published by the Free
+     Software Foundation; either version 3 of the License, or (at your
+     option) any later version.
+
+ or
+
+   * the GNU General Public License as published by the Free Software
+     Foundation; either version 2 of the License, or (at your option) any
+     later version.
+
+ or both in parallel, as here.
+
+ The Bitstream Library is distributed in the hope that it will be useful, but
+ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
+ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
+ for more details.
+
+ You should have received copies of the GNU General Public License and the
+ GNU Lesser General Public License along with the GNU MP Library.  If not,
+ see https://www.gnu.org/licenses/.
+ *******************************************************/
 
 {
 /* state = 0x0 (0 bits, 0x0 buffer) */
 {{0, 0x2},{0, 0x3}},
-/* state = 0x1 (0 bits, 0x1 buffer) */
-{{0, 0x6},{0, 0x7}},
+/* state = 0x1 (0 bits, 0x0 buffer) */
+{{0, 0x2},{0, 0x3}},
 /* state = 0x2 (1 bits, 0x0 buffer) */
 {{0, 0x4},{0, 0x5}},
 /* state = 0x3 (1 bits, 0x1 buffer) */
