@@ -1,7 +1,7 @@
 #!/usr/bin/python
 
 # Audio Tools, a module and set of tools for manipulating audio data
-# Copyright (C) 2007-2014  Brian Langenberger
+# Copyright (C) 2007-2015  Brian Langenberger
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -44,13 +44,21 @@ class FrameListReader:
         self.samples = samples[:]
         self.sample_rate = sample_rate
         self.channels = channels
-        if (channel_mask is None):
-            self.channel_mask = audiotools.ChannelMask.from_channels(channels)
+        if channel_mask is None:
+            self.channel_mask = \
+                int(audiotools.ChannelMask.from_channels(channels))
         else:
             self.channel_mask = channel_mask
         self.bits_per_sample = bits_per_sample
+        self.read = self.read_opened
 
-    def read(self, pcm_frames):
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
+    def read_opened(self, pcm_frames):
         (framelist, self.framelist) = self.framelist.split(pcm_frames)
         return framelist
 
@@ -59,6 +67,7 @@ class FrameListReader:
                                                   self.channels,
                                                   self.bits_per_sample,
                                                   True)
+        self.read = self.read_opened
 
     def read_closed(self, pcm_frames):
         raise ValueError()
@@ -67,17 +76,19 @@ class FrameListReader:
         self.read = self.read_closed
 
 
-class MD5Reader:
+class MD5Reader(audiotools.PCMReader):
     def __init__(self, pcmreader):
+        audiotools.PCMReader.__init__(
+            self,
+            sample_rate=pcmreader.sample_rate,
+            channels=pcmreader.channels,
+            channel_mask=pcmreader.channel_mask,
+            bits_per_sample=pcmreader.bits_per_sample)
         self.pcmreader = pcmreader
-        self.sample_rate = pcmreader.sample_rate
-        self.channels = pcmreader.channels
-        self.channel_mask = pcmreader.channel_mask
-        self.bits_per_sample = pcmreader.bits_per_sample
         self.md5 = md5()
 
     def reset(self):
-        if (hasattr(self.pcmreader, "reset")):
+        if hasattr(self.pcmreader, "reset"):
             self.pcmreader.reset()
         self.md5 = md5()
 
@@ -198,6 +209,7 @@ class Silence24_Mono(Silence8_Mono):
                             bits_per_sample=24)
         self.pcm_frames = pcm_frames
         self.md5 = md5()
+
     def __repr__(self):
         return "Silence24_Mono(%s, %s)" % \
             (repr(self.pcm_frames),
@@ -428,20 +440,22 @@ class Sine24_Stereo(Sine8_Stereo):
              repr(self.fmult))
 
 
-class Simple_Sine:
+class Simple_Sine(audiotools.PCMReader):
     def __init__(self, pcm_frames, sample_rate, channel_mask,
                  bits_per_sample, *values):
+        audiotools.PCMReader.__init__(
+            self,
+            sample_rate=sample_rate,
+            channels=len(values),
+            channel_mask=channel_mask,
+            bits_per_sample=bits_per_sample)
+
         self.pcm_frames = pcm_frames
         self.total_frames = pcm_frames
         self.i = 0
         self.channel_max_values = [v[0] for v in values]
         self.channel_counts = [v[1] for v in values]
 
-        self.sample_rate = sample_rate
-        self.channels = len(values)
-        self.channel_mask = channel_mask
-        self.bits_per_sample = bits_per_sample
-
         self.streams = [Sine_Simple(pcm_frames,
                                     bits_per_sample,
                                     sample_rate,
@@ -493,10 +507,17 @@ class WastedBPS16:
         self.channel_mask = 0x3
         self.bits_per_sample = 16
         self.signed = True
-        self.sample_frame = audiotools.pcm.FrameList("", 2, 16, False, False)
+        self.sample_frame = audiotools.pcm.empty_framelist(2, 16)
         self.md5 = md5()
+        self.read = self.read_opened
 
-    def read(self, pcm_frames):
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.close()
+
+    def read_opened(self, pcm_frames):
         wave = []
         for i in range(min(pcm_frames, self.pcm_frames)):
             wave.append((self.i % 2000) << 2)
@@ -515,6 +536,7 @@ class WastedBPS16:
         raise ValueError()
 
     def reset(self):
+        self.read = self.read_opened
         self.i = 0
         self.pcm_frames = self.total_frames
         self.md5 = md5()
@@ -534,10 +556,13 @@ class WastedBPS16:
 
 class Raw(audiotools.PCMReader):
     def __init__(self, pcm_frames, channels, bits_per_sample):
-        self.sample_rate = 44100
-        self.channels = channels
-        self.bits_per_sample = bits_per_sample
-        self.process = None
+        audiotools.PCMReader.__init__(
+            self,
+            sample_rate=44100,
+            channels=channels,
+            channel_mask=0,
+            bits_per_sample=bits_per_sample)
+
         self.file = BytesIO()
 
         full_scale = (1 << (bits_per_sample - 1)) - 1
